@@ -1134,7 +1134,8 @@ CollectWindow *collection_window_new(const gchar *path)
 	gtk_window_set_wmclass(GTK_WINDOW(cw->window), "collection", "GQview");
 	gtk_container_border_width (GTK_CONTAINER (cw->window), 0);
 
-	gtk_signal_connect(GTK_OBJECT(cw->window), "delete_event", (GtkSignalFunc) collection_window_delete, cw);
+	gtk_signal_connect(GTK_OBJECT(cw->window), "delete_event",
+			   GTK_SIGNAL_FUNC(collection_window_delete), cw);
 
 	gtk_signal_connect(GTK_OBJECT(cw->window),"key_press_event",
 			   GTK_SIGNAL_FUNC(collection_window_keypress), cw);
