@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -14,6 +14,20 @@
 #define DND_H
 
 
+enum {
+	TARGET_APP_COLLECTION_MEMBER,
+	TARGET_URI_LIST,
+	TARGET_TEXT_PLAIN
+};
+
+
+extern GtkTargetEntry dnd_file_drag_types[];
+extern gint dnd_file_drag_types_count;
+
+extern GtkTargetEntry dnd_file_drop_types[];
+extern gint dnd_file_drop_types_count;
+
+
 GList *get_uri_file_list(gchar *data);
 gchar *make_uri_file_list(GList *list, gint plain_text, gint *length);
 
@@ -22,3 +36,4 @@ void init_dnd(void);
 
 
 #endif
+
