@@ -20,6 +20,7 @@
 #include "filelist.h"
 #include "image-load.h"
 #include "img-view.h"
+#include "info.h"
 #include "layout.h"
 #include "layout_image.h"
 #include "menu.h"
@@ -32,9 +33,23 @@
 #include <gdk/gdkkeysyms.h> /* for keyboard values */
 
 
+#include <utime.h>
+
+
 #define DUPE_DEF_WIDTH 600
 #define DUPE_DEF_HEIGHT 400
 
+/* column assignment order (simply change them here) */
+#define DUPE_COLUMN_RANK	0
+#define DUPE_COLUMN_THUMB	1
+#define DUPE_COLUMN_NAME	2
+#define DUPE_COLUMN_SIZE	3
+#define DUPE_COLUMN_DATE	4
+#define DUPE_COLUMN_DIMENSIONS	5
+#define DUPE_COLUMN_PATH	6
+
+#define DUPE_COLUMN_COUNT	7	/* total columns */
+
 
 static GList *dupe_window_list = NULL;	/* list of open DupeWindow *s */
 
@@ -271,7 +286,7 @@ static void dupe_item_read_cache(DupeItem *di)
 	path = cache_find_location(di->path, GQVIEW_CACHE_SIMIL_EXT);
 	if (!path) return;
 
-	if (filetime(di->path) > filetime(path))
+	if (filetime(di->path) != filetime(path))
 		{
 		g_free(path);
 		return;
@@ -319,7 +334,17 @@ static void dupe_item_write_cache(DupeItem *di)
 		if (di->checksum != 0) cache_sim_data_set_checksum(cd, di->checksum);
 		if (di->simd) cache_sim_data_set_similarity(cd, di->simd);
 
-		cache_sim_data_save(cd);
+		if (cache_sim_data_save(cd))
+			{
+			struct utimbuf ut;
+			/* set time to that of source file */
+
+			ut.actime = ut.modtime = filetime(di->path);
+ 			if (ut.modtime > 0)
+ 				{
+ 				utime(cd->path, &ut);
+ 				}
+			}
 		cache_sim_data_free(cd);
 		}
 	g_free(base);
@@ -343,7 +368,7 @@ static void dupe_data_free(DupeData *dd)
 static void dupe_data_add(DupeWindow *dw, DupeData *dd, DupeItem *di)
 {
 	gint row;
-	gchar *text[8];
+	gchar *text[9];
 	GtkStyle *style = NULL;
 
 	if (di->dupe) return;
@@ -356,6 +381,7 @@ static void dupe_data_add(DupeWindow *dw, DupeData *dd, DupeItem *di)
 		row = gtk_clist_find_row_from_data(GTK_CLIST(dw->clist), (GList *)(dd->list->next)->data);
 		style = gtk_clist_get_row_style(GTK_CLIST(dw->clist), row);
 		if (style) gtk_style_ref(style);
+		row++;
 		}
 	else
 		{
@@ -363,23 +389,33 @@ static void dupe_data_add(DupeWindow *dw, DupeData *dd, DupeItem *di)
 		style = dupe_clist_new_match_style(dw, row);
 		}
 
-	text[0] = "";
-	text[1] = (gchar *)di->name;
-	text[2] = text_from_size(di->size);
-	text[3] = (gchar *)text_from_time(di->date);
-	if (di->width > 0 && di->height > 0)
+	if (di->rank == 0 || !dd->list->next)
 		{
-		text[4] = g_strdup_printf("%d x %d", di->width, di->height);
+		text[DUPE_COLUMN_RANK] = g_strdup("");
 		}
 	else
 		{
-		text[4] = g_strdup("");
+		text[DUPE_COLUMN_RANK] = g_strdup_printf("%d", di->rank);
 		}
-	text[5] = di->path;
-	text[6] = NULL;
+
+	text[DUPE_COLUMN_THUMB] = "";
+	text[DUPE_COLUMN_NAME] = (gchar *)di->name;
+	text[DUPE_COLUMN_SIZE] = text_from_size(di->size);
+	text[DUPE_COLUMN_DATE] = (gchar *)text_from_time(di->date);
+	if (di->width > 0 && di->height > 0)
+		{
+		text[DUPE_COLUMN_DIMENSIONS] = g_strdup_printf("%d x %d", di->width, di->height);
+		}
+	else
+		{
+		text[DUPE_COLUMN_DIMENSIONS] = g_strdup("");
+		}
+	text[DUPE_COLUMN_PATH] = di->path;
+	text[DUPE_COLUMN_COUNT] = NULL;
 	row = gtk_clist_insert(GTK_CLIST(dw->clist), row, text);
-	g_free(text[2]);
-	g_free(text[4]);
+	g_free(text[DUPE_COLUMN_RANK]);
+	g_free(text[DUPE_COLUMN_SIZE]);
+	g_free(text[DUPE_COLUMN_DIMENSIONS]);
 
 	if (style)
 		{
@@ -476,58 +512,96 @@ static gint dupe_match(DupeItem *a, DupeItem *b, DupeMatchType mask)
 	if (mask & DUPE_MATCH_PATH)
 		{
 		if (strcmp(a->path, b->path) == 0)
+			{
 			match = 1;
+			a->rank = 0;
+			b->rank = 0;
+			}
 		else
+			{
 			return FALSE;
+			}
 		}
 	if (mask & DUPE_MATCH_NAME)
 		{
 		if (strcmp(a->name, b->name) == 0)
+			{
 			match = 1;
+			a->rank = 0;
+			b->rank = 0;
+			}
 		else
+			{
 			return FALSE;
+			}
 		}
 	if (mask & DUPE_MATCH_SIZE)
 		{
 		if (a->size == b->size)
+			{
 			match = 1;
+			a->rank = 0;
+			b->rank = 0;
+			}
 		else
+			{
 			return FALSE;
+			}
 		}
 	if (mask & DUPE_MATCH_DATE)
 		{
 		if (a->date == b->date)
+			{
 			match = 1;
+			a->rank = 0;
+			b->rank = 0;
+			}
 		else
+			{
 			return FALSE;
+			}
 		}
 	if (mask & DUPE_MATCH_SUM)
 		{
 		if (a->checksum == 0) a->checksum = checksum_simple(a->path);
 		if (b->checksum == 0) b->checksum = checksum_simple(b->path);
 		if (a->checksum == b->checksum)
+			{
 			match = 1;
+			a->rank = 0;
+			b->rank = 0;
+			}
 		else
+			{
 			return FALSE;
+			}
 		}
 	if (mask & DUPE_MATCH_DIM)
 		{
 		if (a->width == 0) image_load_dimensions(a->path, &a->width, &a->height);
 		if (b->width == 0) image_load_dimensions(b->path, &b->width, &b->height);
 		if (a->width == b->width && a->height == b->height)
+			{
 			match = 1;
+			a->rank = 0;
+			b->rank = 0;
+			}
 		else
+			{
 			return FALSE;
+			}
 		}
 	if (mask & DUPE_MATCH_SIM_HIGH ||
 	    mask & DUPE_MATCH_SIM_MED ||
-	    mask & DUPE_MATCH_SIM_LOW)
+	    mask & DUPE_MATCH_SIM_LOW ||
+	    mask & DUPE_MATCH_SIM_CUSTOM)
 		{
 		gfloat f;
 		gfloat m;
 
 		if (mask & DUPE_MATCH_SIM_HIGH) m = 0.95;
 		else if (mask & DUPE_MATCH_SIM_MED) m = 0.90;
+		else if (mask & DUPE_MATCH_SIM_CUSTOM) m = (float)dupe_custom_threshold / 100.0;
 		else m = 0.85;
 
 		f = image_sim_compare_fast(a->simd, b->simd, m);
@@ -536,9 +610,12 @@ static gint dupe_match(DupeItem *a, DupeItem *b, DupeMatchType mask)
 			{
 			if (debug) printf("similar: %32s %32s = %f\n", a->name, b->name, f);
 			match = 1;
+			a->rank = b->rank = (gint)(f * 100.0);
 			}
 		else
+			{
 			return FALSE;
+			}
 		}
 
 	return match;
@@ -582,8 +659,8 @@ static void dupe_list_check_match(DupeWindow *dw, DupeItem *needle, GList *start
 				dd = dupe_data_new();
 				dw->dupes = g_list_prepend(dw->dupes, dd);
 				}
-			dupe_data_add(dw, dd, di);
 			dupe_data_add(dw, dd, needle);
+			dupe_data_add(dw, dd, di);
 			}
 		}
 }
@@ -595,7 +672,7 @@ static void dupe_clist_set_thumb(DupeWindow *dw, DupeItem *di)
 	row = gtk_clist_find_row_from_data(GTK_CLIST(dw->clist), di);
 	if (row < 0) return;
 
-	gtk_clist_set_pixmap(GTK_CLIST(dw->clist), row, 0, di->pixmap, di->mask);
+	gtk_clist_set_pixmap(GTK_CLIST(dw->clist), row, DUPE_COLUMN_THUMB, di->pixmap, di->mask);
 }
 
 static void dupe_thumb_do(DupeWindow *dw)
@@ -642,7 +719,7 @@ static void dupe_thumb_step(DupeWindow *dw)
 
 	while (!di && i < length)
 		{
-		if (!gtk_clist_get_pixmap(GTK_CLIST(dw->clist), i, 0, NULL, NULL))
+		if (!gtk_clist_get_pixmap(GTK_CLIST(dw->clist), i, DUPE_COLUMN_THUMB, NULL, NULL))
 			{
 			di = gtk_clist_get_row_data(GTK_CLIST(dw->clist), i);
 			
@@ -828,7 +905,8 @@ static gint dupe_check_cb(gpointer data)
 			}
 		if ((dw->match_mask & DUPE_MATCH_SIM_HIGH ||
 		     dw->match_mask & DUPE_MATCH_SIM_MED ||
-		     dw->match_mask & DUPE_MATCH_SIM_LOW) &&
+		     dw->match_mask & DUPE_MATCH_SIM_LOW ||
+		     dw->match_mask & DUPE_MATCH_SIM_CUSTOM) &&
 		    !(dw->setup_mask & DUPE_MATCH_SIM_MED) )
 			{
 			if (!dw->setup_point) dw->setup_point = dw->list;
@@ -1008,8 +1086,8 @@ static gint dupe_item_set_path(DupeWindow *dw, const gchar *source, const gchar
 		row = gtk_clist_find_row_from_data(GTK_CLIST(dw->clist), di);
 		if (row >= 0)
 			{
-			gtk_clist_set_text(GTK_CLIST(dw->clist), row, 1, di->name);
-			gtk_clist_set_text(GTK_CLIST(dw->clist), row, 5, di->path);
+			gtk_clist_set_text(GTK_CLIST(dw->clist), row, DUPE_COLUMN_NAME, di->name);
+			gtk_clist_set_text(GTK_CLIST(dw->clist), row, DUPE_COLUMN_PATH, di->path);
 			}
 		}
 
@@ -1331,6 +1409,13 @@ static void dupe_menu_edit_cb(GtkWidget *widget, gpointer data)
 	dupe_window_edit_selected(dw, n);
 }
 
+static void dupe_menu_info_cb(GtkWidget *widget, gpointer data)
+{
+	DupeWindow *dw = data;
+
+	info_window_new(NULL, dupe_clist_get_selected(dw, dw->clist));
+}
+
 static void dupe_menu_collection_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
@@ -1387,7 +1472,7 @@ static void dupe_menu_close_cb(GtkWidget *widget, gpointer data)
 	dupe_window_close(dw);
 }
 
-static void dupe_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint dupe_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	DupeWindow *dw = data;
 
@@ -1415,6 +1500,8 @@ static void dupe_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpoin
 		menu_item_add_divider(menu);
 		submenu_add_edit(menu, &item, GTK_SIGNAL_FUNC(dupe_menu_edit_cb), dw);
 		if (row < 0) gtk_widget_set_sensitive(item, FALSE);
+		item = menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(dupe_menu_info_cb), dw);
+		if (row < 0) gtk_widget_set_sensitive(item, FALSE);
 		item = menu_item_add(menu, _("Add to new collection"), GTK_SIGNAL_FUNC(dupe_menu_collection_cb), dw);
 		if (row < 0) gtk_widget_set_sensitive(item, FALSE);
 		menu_item_add_divider(menu);
@@ -1437,7 +1524,7 @@ static void dupe_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpoin
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, bevent->button, bevent->time);
 		}
 
-	if (row < 0 || col < 0) return;
+	if (row < 0 || col < 0) return FALSE;
 
 	dw->click_row = row;
 
@@ -1454,6 +1541,8 @@ static void dupe_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpoin
 			gtk_clist_select_row(GTK_CLIST(dw->clist), row, col);
 			}
 		}
+
+	return FALSE;
 }
 
 /*
@@ -1559,7 +1648,7 @@ static void dupe_second_menu_clear_cb(GtkWidget *widget, gpointer data)
 	dupe_window_recompare(dw);
 }
 
-static void dupe_second_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint dupe_second_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	DupeWindow *dw = data;
 
@@ -1595,7 +1684,7 @@ static void dupe_second_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, bevent->button, bevent->time);
 		}
 
-	if (row < 0 || col < 0) return;
+	if (row < 0 || col < 0) return FALSE;
 
 	dw->click_row = row;
 
@@ -1612,6 +1701,8 @@ static void dupe_second_clist_press_cb(GtkWidget *widget, GdkEventButton *bevent
 			gtk_clist_select_row(GTK_CLIST(dw->second_clist), row, col);
 			}
 		}
+
+	return FALSE;
 }
 
 static void dupe_second_set_toggle_cb(GtkWidget *widget, gpointer data)
@@ -1657,7 +1748,8 @@ static void dupe_menu_add_item(GtkWidget *menu, const gchar *text, DupeMatchType
 
 	item = gtk_menu_item_new_with_label(text);
 	gtk_object_set_data(GTK_OBJECT(item), "dupe_type", GINT_TO_POINTER(type));
-	gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(dupe_menu_type_cb), dw);
+	gtk_signal_connect(GTK_OBJECT(item), "activate",
+			   GTK_SIGNAL_FUNC(dupe_menu_type_cb), dw);
 	gtk_menu_append(GTK_MENU(menu), item);
 	gtk_widget_show(item);
 }
@@ -1678,6 +1770,7 @@ static void dupe_menu_setup(DupeWindow *dw)
 	dupe_menu_add_item(menu, _("Similarity (high)"), DUPE_MATCH_SIM_HIGH, dw);
 	dupe_menu_add_item(menu, _("Similarity"), DUPE_MATCH_SIM_MED, dw);
 	dupe_menu_add_item(menu, _("Similarity (low)"), DUPE_MATCH_SIM_LOW, dw);
+	dupe_menu_add_item(menu, _("Similarity (custom)"), DUPE_MATCH_SIM_CUSTOM, dw);
 
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(dw->dupe_omenu), menu);
 
@@ -1710,6 +1803,9 @@ static void dupe_menu_setup(DupeWindow *dw)
 		case DUPE_MATCH_SIM_LOW:
 			n = 8;
 			break;
+		case DUPE_MATCH_SIM_CUSTOM:
+			n = 9;
+			break;
 		default:
 			n = 0;
 			break;
@@ -1746,7 +1842,7 @@ static void dupe_window_show_thumb_cb(GtkWidget *widget, gpointer data)
 		length = GTK_CLIST(dw->clist)->rows;
 		for (i = 0; i < length; i++)
 			{
-			gtk_clist_set_text(GTK_CLIST(dw->clist), i, 0, "");
+			gtk_clist_set_text(GTK_CLIST(dw->clist), i, DUPE_COLUMN_THUMB, "");
 			}
 		gtk_clist_thaw(GTK_CLIST(dw->clist));
 		dupe_window_update_progress(dw, NULL, 0.0);
@@ -1939,10 +2035,12 @@ void dupe_window_close(DupeWindow *dw)
 	g_free(dw);
 }
 
-static void dupe_window_destroy(GtkWidget *widget, GdkEvent *event, gpointer data)
+static gint dupe_window_delete(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
 	DupeWindow *dw = data;
 	dupe_window_close(dw);
+
+	return TRUE;
 }
 
 /* collection and files can be NULL */
@@ -1957,6 +2055,7 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 	GtkWidget *button;
 	GtkWidget *sep;
 	gchar *title[] = {
+		"",
 		"",
 		_("Name"),
 		_("Size"),
@@ -1990,7 +2089,7 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
         gtk_container_border_width (GTK_CONTAINER (dw->window), 0);
 
         gtk_signal_connect(GTK_OBJECT(dw->window), "delete_event",
-			   GTK_SIGNAL_FUNC(dupe_window_destroy), dw);
+			   GTK_SIGNAL_FUNC(dupe_window_delete), dw);
 	gtk_signal_connect(GTK_OBJECT(dw->window), "key_press_event",
 			   GTK_SIGNAL_FUNC(dupe_window_keypress_cb), dw);
 
@@ -2007,17 +2106,18 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 	gtk_table_attach_defaults(GTK_TABLE(dw->table), scrolled, 0, 2, 0, 1);
         gtk_widget_show(scrolled);
 
-	dw->clist = gtk_clist_new_with_titles(6, title);
+	dw->clist = gtk_clist_new_with_titles(DUPE_COLUMN_COUNT, title);
 	gtk_clist_set_selection_mode(GTK_CLIST(dw->clist), GTK_SELECTION_EXTENDED);
 	
-	for (i = 0; i < 6; i++)
+	for (i = 0; i < DUPE_COLUMN_COUNT; i++)
 		{
 		gtk_clist_set_column_auto_resize(GTK_CLIST(dw->clist),  i, TRUE);
 		}
-	gtk_clist_set_column_justification(GTK_CLIST(dw->clist), 2, GTK_JUSTIFY_RIGHT);
-	gtk_clist_set_column_justification(GTK_CLIST(dw->clist), 3, GTK_JUSTIFY_RIGHT);
-	gtk_clist_set_column_justification(GTK_CLIST(dw->clist), 4, GTK_JUSTIFY_CENTER);
-	gtk_signal_connect(GTK_OBJECT(dw->clist), "button_press_event", GTK_SIGNAL_FUNC(dupe_clist_press_cb), dw);
+	gtk_clist_set_column_justification(GTK_CLIST(dw->clist), DUPE_COLUMN_SIZE, GTK_JUSTIFY_RIGHT);
+	gtk_clist_set_column_justification(GTK_CLIST(dw->clist), DUPE_COLUMN_DATE, GTK_JUSTIFY_RIGHT);
+	gtk_clist_set_column_justification(GTK_CLIST(dw->clist), DUPE_COLUMN_DIMENSIONS, GTK_JUSTIFY_CENTER);
+	gtk_signal_connect(GTK_OBJECT(dw->clist), "button_press_event",
+			   GTK_SIGNAL_FUNC(dupe_clist_press_cb), dw);
 	gtk_container_add(GTK_CONTAINER(scrolled), dw->clist);
 	if (dw->show_thumbs) gtk_clist_set_row_height(GTK_CLIST(dw->clist), thumb_max_height);
         gtk_widget_show(dw->clist);
@@ -2046,7 +2146,8 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 		{
 		gtk_clist_set_column_auto_resize(GTK_CLIST(dw->second_clist),  i, TRUE);
 		}
-	gtk_signal_connect(GTK_OBJECT(dw->second_clist), "button_press_event", GTK_SIGNAL_FUNC(dupe_second_clist_press_cb), dw);
+	gtk_signal_connect(GTK_OBJECT(dw->second_clist), "button_press_event",
+			   GTK_SIGNAL_FUNC(dupe_second_clist_press_cb), dw);
 	gtk_container_add(GTK_CONTAINER(scrolled), dw->second_clist);
         gtk_widget_show(dw->second_clist);
 
@@ -2073,13 +2174,15 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 
 	button = gtk_check_button_new_with_label(_("Thumbnails"));
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), dw->show_thumbs);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(dupe_window_show_thumb_cb), dw);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(dupe_window_show_thumb_cb), dw);
 	gtk_box_pack_start(GTK_BOX(status_box), button, FALSE, FALSE, 5);
 	gtk_widget_show(button);
 
 	button = gtk_check_button_new_with_label(_("Compare two file sets"));
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), dw->second_set);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(dupe_second_set_toggle_cb), dw);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(dupe_second_set_toggle_cb), dw);
 	gtk_box_pack_end(GTK_BOX(status_box), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
@@ -2185,15 +2288,16 @@ static GtkWidget *dupe_confirm_dir_list(DupeWindow *dw, GList *list)
 	d->list = list;
 
 	menu = popup_menu_short_lived();
-	gtk_signal_connect(GTK_OBJECT(menu), "destroy", confirm_dir_list_destroy, d);
+	gtk_signal_connect(GTK_OBJECT(menu), "destroy",
+			   GTK_SIGNAL_FUNC(confirm_dir_list_destroy), d);
 
 	menu_item_add(menu, _("Dropped list includes directories."), NULL, NULL);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Add contents"), confirm_dir_list_add, d);
-	menu_item_add(menu, _("Add contents recursive"), confirm_dir_list_recurse, d);
-	menu_item_add(menu, _("Skip directories"), confirm_dir_list_skip, d);
+	menu_item_add(menu, _("Add contents"), GTK_SIGNAL_FUNC(confirm_dir_list_add), d);
+	menu_item_add(menu, _("Add contents recursive"), GTK_SIGNAL_FUNC(confirm_dir_list_recurse), d);
+	menu_item_add(menu, _("Skip directories"), GTK_SIGNAL_FUNC(confirm_dir_list_skip), d);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Cancel"), confirm_dir_list_cancel, d);
+	menu_item_add(menu, _("Cancel"), GTK_SIGNAL_FUNC(confirm_dir_list_cancel), d);
 
 	return menu;
 }
@@ -2296,21 +2400,21 @@ static void dupe_dnd_init(DupeWindow *dw)
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 
 	gtk_signal_connect(GTK_OBJECT(dw->clist), "drag_data_get",
-			   dupe_dnd_data_set, dw);
+			   GTK_SIGNAL_FUNC(dupe_dnd_data_set), dw);
 
 	gtk_drag_dest_set(dw->clist,
 			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
 			  dupe_drop_types, n_dupe_drop_types,
 			  GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK);
 	gtk_signal_connect(GTK_OBJECT(dw->clist), "drag_data_received",
-			   dupe_dnd_data_get, dw);
+			   GTK_SIGNAL_FUNC(dupe_dnd_data_get), dw);
 
 	gtk_drag_dest_set(dw->second_clist,
 			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
 			  dupe_drop_types, n_dupe_drop_types,
 			  GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK);
 	gtk_signal_connect(GTK_OBJECT(dw->second_clist), "drag_data_received",
-			   dupe_dnd_data_get, dw);
+			   GTK_SIGNAL_FUNC(dupe_dnd_data_get), dw);
 }
 
 /*
@@ -2348,7 +2452,3 @@ void dupe_maint_renamed(const gchar *source, const gchar *dest)
 
 }
 
-
-
-
-
