@@ -27,7 +27,8 @@ typedef enum
 	DUPE_MATCH_PATH = 1 << 5,
 	DUPE_MATCH_SIM_HIGH = 1 << 6,	/* similarity */
 	DUPE_MATCH_SIM_MED  = 1 << 7,
-	DUPE_MATCH_SIM_LOW  = 1 << 8
+	DUPE_MATCH_SIM_LOW  = 1 << 8,
+	DUPE_MATCH_SIM_CUSTOM = 1 << 9
 } DupeMatchType;
 
 typedef struct _DupeItem DupeItem;
@@ -48,6 +49,7 @@ struct _DupeItem
 	gint height;
 
 	ImageSimilarityData *simd;
+	gint rank;
 
 	/* thumb */
 	GdkPixmap *pixmap;
