@@ -73,12 +73,14 @@ static void fullscreen_hide_mouse_reset(FullScreenData *fs)
 	fs->hide_mouse_id = gtk_timeout_add(FULL_SCREEN_HIDE_MOUSE_DELAY, fullscreen_hide_mouse_cb, fs);
 }
 
-static void fullscreen_mouse_moved(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint fullscreen_mouse_moved(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	FullScreenData *fs = data;
 
 	clear_mouse_cursor(fs->window, FALSE);
 	fullscreen_hide_mouse_reset(fs);
+
+	return FALSE;
 }
 
 static gint fullscreen_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
@@ -112,7 +114,8 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 
 	fs->window = gtk_window_new(GTK_WINDOW_POPUP);
 	gtk_container_border_width(GTK_CONTAINER(fs->window), 0);
-	gtk_signal_connect(GTK_OBJECT(fs->window), "delete_event", (GtkSignalFunc)fullscreen_delete_cb, fs);
+	gtk_signal_connect(GTK_OBJECT(fs->window), "delete_event",
+			   GTK_SIGNAL_FUNC(fullscreen_delete_cb), fs);
 
 	gtk_window_set_title(GTK_WINDOW(fs->window), _("GQview full screen"));
 	gtk_widget_set_usize(fs->window, w, h);
@@ -125,12 +128,7 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 	/* set background to black */
 	if (BLACK_BACKGROUND)
 		{
-		GtkStyle *style;
-		style = gtk_style_copy(gtk_widget_get_style(fs->imd->image));
-		gtk_style_ref(style);
-		style->bg[GTK_STATE_NORMAL] = style->black;
-		gtk_widget_set_style(fs->imd->image, style);
-		gtk_style_unref(style);
+		image_background_set_black(fs->imd, TRUE);
 		}
 
 	gtk_widget_show(fs->imd->widget);
