@@ -61,7 +61,8 @@
  *----------------------------------------------------------------------------
  */
 
-#define GDK_PIXBUF_TILES_BROKEN 1	/* tiles is broken in gdk-pixbuf 0.10.0 */
+/* tiles is broken in gdk-pixbuf <  0.16.0 */
+#define GDK_PIXBUF_TILES_BROKEN  (gdk_pixbuf_major_version == 0 && gdk_pixbuf_minor_version < 16)
 
 #define GQVIEW_RC_DIR             ".gqview"
 #define GQVIEW_RC_DIR_THUMBS      ".gqview/thumbnails"
@@ -98,6 +99,7 @@ extern gint float_window_w;
 extern gint float_window_h;
 extern gint float_window_x;
 extern gint float_window_y;
+extern gint float_window_divider;
 
 extern gint window_hdivider_pos;
 extern gint window_vdivider_pos;
@@ -105,6 +107,7 @@ extern gint window_vdivider_pos;
 extern gint save_window_positions;
 extern gint tools_float;
 extern gint tools_hidden;
+extern gint toolbar_hidden;
 extern gint progressive_key_scrolling;
 
 extern gint startup_path_enable;
@@ -113,6 +116,7 @@ extern gint confirm_delete;
 extern gint enable_delete_key;
 extern gint restore_tool;
 extern gint zoom_mode;
+extern gint two_pass_zoom;
 extern gint fit_window;
 extern gint limit_window_size;
 extern gint zoom_to_fit_expands;
@@ -148,6 +152,10 @@ extern gint slideshow_repeat;
 extern gint mousewheel_scrolls;
 extern gint enable_in_place_rename;
 
+extern gint black_window_background;
+
+extern gint dupe_custom_threshold;
+
 extern gint debug;
 
 extern gint recent_list_max;
