@@ -42,7 +42,14 @@ static void image_loader_area_cb(GdkPixbufLoader *loader,
 
 	if (il->func_area_ready)
 		{
-		if (!il->pixbuf) image_loader_sync_pixbuf(il);
+		if (!il->pixbuf)
+			{
+			image_loader_sync_pixbuf(il);
+			if (!il->pixbuf)
+				{
+				printf("critical: area_ready signal with NULL pixbuf (out of mem?)\n");
+				}
+			}
 		il->func_area_ready(il, x, y, w, h, il->data_area_ready);
 		}
 }
@@ -239,7 +246,8 @@ static gint image_loader_setup(ImageLoader *il)
 		}
 
 	il->loader = gdk_pixbuf_loader_new();
-	gtk_signal_connect(GTK_OBJECT(il->loader), "area_updated", image_loader_area_cb, il);
+	gtk_signal_connect(GTK_OBJECT(il->loader), "area_updated",
+			   GTK_SIGNAL_FUNC(image_loader_area_cb), il);
 
 	return image_loader_begin(il);
 }
