@@ -23,10 +23,6 @@
 #include <gdk/gdkx.h>
 
 
-/* define this to enable debug printing (sorry, peppered throughout file) */
-/* #define IMAGE_DEBUG_ENABLE 1 */
-
-
 #define IMAGE_TILE_SIZE 128
 #define IMAGE_ZOOM_MIN -32.0
 #define IMAGE_ZOOM_MAX 32.0
@@ -34,6 +30,10 @@
 /* size of the image loader buffer (512 bytes x defined number) */
 #define IMAGE_LOAD_BUFFER_COUNT 8
 
+/* define this so that more bytes are read per idle loop on larger images (> 1MB)
+ * #define IMAGE_THROTTLE_LARGER_IMAGES 1
+ */
+
 /* distance to drag mouse to disable image flip */
 #define IMAGE_DRAG_SCROLL_THRESHHOLD 4
 
@@ -145,11 +145,6 @@ static void image_tile_cache_free(ImageWindow *imd, CacheData *cd)
 		}
 	if (cd->pixbuf)
 		{
-
-#ifdef IMAGE_DEBUG_ENABLE
-		printf("tile pixbuf free: %p\n", cd->it->pixbuf);
-#endif
-
 		gdk_pixbuf_unref(cd->it->pixbuf);
 		cd->it->pixbuf = NULL;
 		}
@@ -215,10 +210,6 @@ static void image_tile_prepare(ImageWindow *imd, ImageTile *it)
 		size = gdk_pixbuf_get_rowstride(pixbuf) * imd->tile_height;
 		image_tile_cache_free_space(imd, size, it);
 
-#ifdef IMAGE_DEBUG_ENABLE
-		printf("tile pixbuf new : %p\n", pixbuf);
-#endif
-
 		it->pixbuf = pixbuf;
 		image_tile_cache_add(imd, it, NULL, pixbuf, size);
 		}
@@ -342,7 +333,9 @@ static void image_tile_sync(ImageWindow *imd, gint width, gint height, gint blan
 	image_queue_clear(imd);
 }
 
-static void image_tile_render(ImageWindow *imd, ImageTile *it, gint x, gint y, gint w, gint h, gint new_data)
+static void image_tile_render(ImageWindow *imd, ImageTile *it,
+			      gint x, gint y, gint w, gint h,
+			      gint new_data, gint fast)
 {
 	gint has_alpha;
 	gint draw = FALSE;
@@ -355,11 +348,11 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it, gint x, gint y, g
 		y = 0;
 		w = it->w;
 		h = it->h;
-		it->render_done = TILE_RENDER_ALL;
+		if (!fast) it->render_done = TILE_RENDER_ALL;
 		}
 	else if (it->render_todo != TILE_RENDER_AREA)
 		{
-		it->render_todo = TILE_RENDER_NONE;
+		if (!fast) it->render_todo = TILE_RENDER_NONE;
 		return;
 		}
 
@@ -368,11 +361,6 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it, gint x, gint y, g
 	image_tile_prepare(imd, it);
 	has_alpha = gdk_pixbuf_get_has_alpha(imd->pixbuf);
 
-#ifdef IMAGE_DEBUG_ENABLE
-	printf("image.c: rendering tile at %d, %d (%d x %d) [dim: %d x %d] nd:%d\n", it->x + x, it->y + y, w, h, imd->width, imd->height, new_data);
-	printf("image.c: rendered  tile is %d, %d (%d x %d)\n", it->x, it->y, it->w, it->h);
-#endif
-
 	/* FIXME checker colors for alpha should be configurable,
 	 * also should be drawn for blank = TRUE
 	 */
@@ -409,54 +397,36 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it, gint x, gint y, g
 	else
 		{
 		double scale_x, scale_y;
+		gint simple;
 
 		if (imd->image_width == 0 || imd->image_height == 0) return;
 		scale_x = (double)imd->width / imd->image_width;
 		scale_y = (double)imd->height / imd->image_height;
 
-#ifdef IMAGE_DEBUG_ENABLE
-		printf("image.c (render scaled): %d , %d (%d x %d) offset %d, %d (%f : %f) [%d, %d]\n",
-			x, y, w, h, 0 - it->x, 0 - it->y, scale_x, scale_y, it->x, it->y);
-		printf("image.c (extras): src %d x %d dst %d x %d\n",
-			gdk_pixbuf_get_width(imd->pixbuf), gdk_pixbuf_get_height(imd->pixbuf),
-			gdk_pixbuf_get_width(it->pixbuf), gdk_pixbuf_get_height(it->pixbuf));
-#endif
+		if (GDK_PIXBUF_TILES_BROKEN)
+			{
+			simple = (!has_alpha && (GdkInterpType)zoom_quality == GDK_INTERP_NEAREST);
+			}
+		else
+			{
+			simple = (!has_alpha);
+			}
 
-/* I don't know, this just seems to break whenever the tile is for the right side of the image
- * (note that GDK_INTERP_NEAREST does not show the brokenness ?
- *
- * UPDATE:
- * Ok, here is the deal: gdk_pixbuf_scale() is broken for anything but INTERP_NEAREST,
- * and gdk_pixbuf_composite_color() is only broken for INTERP_TILES (my favorite render type, btw)
- * so here is what we are gonna do: use gdk_pixbuf_composite_color(), and complain to gdk-pixbuf authors.
- *
- * UPDATE2: even gdk_pixbuf_composite_color() can crash under certain circumstances, maybe I have a bug
- * in here, somewhere. (Although GDK_INTERP_NEAREST never crashes with the same image and zoom/scale).
- *
- * UPDATE3: well, with gdk-pixbuf 0.10.0 it looks like some of the problems were fixed, only TILES
- * still appears to crash, in fact it now crashes in composite_color() too, so INTERP_TILES support
- * is dropped till it is fixed.
- *
- * UPDATE4: SCRATCH-THAT! gdk_pixbuf_scale is still broke! BAAH!
- */
-
-#ifdef GDK_PIXBUF_TILES_BROKEN
-		if (!has_alpha && (GdkInterpType)zoom_quality == GDK_INTERP_NEAREST)
-#else
-		if (!has_alpha)
-#endif
+		if (simple)
 			{
 			gdk_pixbuf_scale(imd->pixbuf, it->pixbuf, x, y, w, h,
 					 (double) 0.0 - it->x,
 					 (double) 0.0 - it->y,
-					 scale_x, scale_y, (GdkInterpType)zoom_quality);
+					 scale_x, scale_y,
+					 (fast) ? GDK_INTERP_NEAREST : (GdkInterpType)zoom_quality);
 			}
 		else
 			{
 			gdk_pixbuf_composite_color(imd->pixbuf, it->pixbuf, x, y, w, h,
 					 (double) 0.0 - it->x,
 					 (double) 0.0 - it->y,
-					 scale_x, scale_y, (GdkInterpType)zoom_quality,
+					 scale_x, scale_y,
+					 (fast) ? GDK_INTERP_NEAREST : (GdkInterpType)zoom_quality,
 					 255, it->x + x, it->y + y,
 					 IMAGE_ALPHA_CHECK_SIZE, IMAGE_ALPHA_CHECK1, IMAGE_ALPHA_CHECK2);
 			}
@@ -473,16 +443,14 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it, gint x, gint y, g
 					      (GdkRgbDither)dither_quality, it->x + x, it->y + y);
 		}
 
-	it->render_todo = TILE_RENDER_NONE;
+	if (!fast) it->render_todo = TILE_RENDER_NONE;
 }
 
-static void image_tile_expose(ImageWindow *imd, ImageTile *it, gint x, gint y, gint w, gint h, gint new_data)
+static void image_tile_expose(ImageWindow *imd, ImageTile *it,
+			      gint x, gint y, gint w, gint h,
+			      gint new_data, gint fast)
 {
-	image_tile_render(imd, it, x, y, w, h, new_data);
-
-#ifdef IMAGE_DEBUG_ENABLE
-	if (debug) printf("expose (%d, %d): +%d, +%d (%d x %d)\n", it->x, it->y, x, y, w, h);
-#endif
+	image_tile_render(imd, it, x, y, w, h, new_data, fast);
 
 	gdk_draw_pixmap (imd->image->window, imd->image->style->fg_gc[GTK_WIDGET_STATE(imd->image)],
 			 it->pixmap, x, y,
@@ -506,25 +474,39 @@ static gint image_queue_draw_idle_cb(gpointer data)
 {
 	ImageWindow *imd = data;
 	QueueData *qd;
+	gint fast;
 
-	if (!imd->pixbuf || !imd->draw_queue || imd->draw_idle_id == -1)
+	if (!imd->pixbuf || (!imd->draw_queue && !imd->draw_queue_2pass) || imd->draw_idle_id == -1)
 		{
-
-#ifdef IMAGE_DEBUG_ENABLE
-		if (debug) printf("queue: abort\n");
-#endif
-
 		imd->draw_idle_id = -1;
 		return FALSE;
 		}
 
-	qd = imd->draw_queue->data;
+	if (imd->draw_queue)
+		{
+		qd = imd->draw_queue->data;
+		fast = (two_pass_zoom && (GdkInterpType)zoom_quality != GDK_INTERP_NEAREST && imd->scale != 1.0);
+		}
+	else
+		{
+		if (imd->il)
+			{
+			/* still loading, wait till done (also drops the higher priority) */
+
+			imd->draw_idle_id = gtk_idle_add_priority(G_PRIORITY_DEFAULT_IDLE,
+								  image_queue_draw_idle_cb, imd);
+			imd->draw_idle_high = FALSE;
+			return FALSE;
+			}
+		qd = imd->draw_queue_2pass->data;
+		fast = FALSE;
+		}
 
 	if (GTK_WIDGET_REALIZED(imd->image))
 		{
 		if (image_tile_is_visible(imd, qd->it))
 			{
-			image_tile_expose(imd, qd->it, qd->x, qd->y, qd->w, qd->h, qd->new_data);
+			image_tile_expose(imd, qd->it, qd->x, qd->y, qd->w, qd->h, qd->new_data, fast);
 			}
 		else if (qd->new_data)
 			{
@@ -532,29 +514,35 @@ static gint image_queue_draw_idle_cb(gpointer data)
 			qd->it->blank = FALSE;
 			if (qd->it->pixmap && qd->it->render_done == TILE_RENDER_ALL)
 				{
-				image_tile_render(imd, qd->it, qd->x, qd->y, qd->w, qd->h, qd->new_data);
+				image_tile_render(imd, qd->it, qd->x, qd->y, qd->w, qd->h, qd->new_data, fast);
 				}
 			}
 		}
 
-	imd->draw_queue = g_list_remove(imd->draw_queue, qd);
-	g_free(qd);
-
-	if (!imd->draw_queue)
+	if (imd->draw_queue)
 		{
+		imd->draw_queue = g_list_remove(imd->draw_queue, qd);
+		if (fast)
+			{
+			imd->draw_queue_2pass = g_list_append(imd->draw_queue_2pass, qd);
+			}
+		else
+			{
+			g_free(qd);
+			}
+		}
+	else
+		{
+		imd->draw_queue_2pass = g_list_remove(imd->draw_queue_2pass, qd);
+		g_free(qd);
+		}
 
-#ifdef IMAGE_DEBUG_ENABLE
-		if (debug) printf("queue: done\n");
-#endif
-
+	if (!imd->draw_queue && !imd->draw_queue_2pass)
+		{
 		imd->draw_idle_id = -1;
 		return FALSE;
 		}
 
-#ifdef IMAGE_DEBUG_ENABLE
-	if (debug) printf("queue: continue\n");
-#endif
-
 	return TRUE;
 }
 
@@ -563,15 +551,8 @@ static QueueData *image_queue_combine(ImageWindow *imd, QueueData *qd)
 	QueueData *found = NULL;
 	GList *work;
 
-#ifdef IMAGE_DEBUG_ENABLE
-	if (qd->it->x + qd->x + qd->w > imd->width || qd->it->y + qd->y + qd->h > imd->height)
-		{
-		printf("WARNING: queue exceeds image bounds!\n");
-		}
-#endif
-
 	work = imd->draw_queue;
-	while(work && !found)
+	while (work && !found)
 		{
 		found = work->data;
 		work = work->next;
@@ -691,36 +672,16 @@ static gint image_queue_to_tiles(ImageWindow *imd, gint x, gint y, gint w, gint
 				qd->h = y + h - j - qd->y;
 				if (qd->y + qd->h > imd->tile_height) qd->h = imd->tile_height - qd->y;
 
-#ifdef IMAGE_DEBUG_ENABLE
-				printf("add tile: [#%3d] [%3d, %3d (%3d x %3d)] %3d, %3d (%3d x %3d) new data:%d\n",
-					g_list_index(imd->tiles, it), it->x, it->y, it->w, it->h,
-					qd->x, qd->y, qd->w, qd->h, qd->new_data);
-#endif
-
 				if (qd->w < 1 || qd->h < 1 || /* <--- sanity checks, rare cases cause this */
 				    image_queue_combine(imd, qd))
 					{
-
-#ifdef IMAGE_DEBUG_ENABLE
-					if (debug) printf("queue combined (%d, %d): +%d, +%d (%d x %d)\n", i, j, qd->x, qd->y, qd->w, qd->h);
-#endif
-
 					g_free(qd);
 					}
 				else
 					{
 					imd->draw_queue = g_list_append(imd->draw_queue, qd);
-
-#ifdef IMAGE_DEBUG_ENABLE
-					if (debug) printf("queue added (%d, %d): +%d, +%d (%d x %d)\n", i, j, qd->x, qd->y, qd->w, qd->h);
-#endif
-
 					}
 
-#ifdef IMAGE_DEBUG_ENABLE
-				if (debug) printf("queued it: (%d, %d)\n", it->x, it->y);
-#endif
-
 				tmp = tmp->next;
 				}
 			}
@@ -744,18 +705,20 @@ static void image_queue(ImageWindow *imd, gint x, gint y, gint w, gint h,
 	if (w < 1 || h < 1) return;
 
 	if (image_queue_to_tiles(imd, x, y, w, h, clamp, render, new_data) &&
-	    (!imd->draw_queue || imd->draw_idle_id == -1))
+	    ((!imd->draw_queue && !imd->draw_queue_2pass) || imd->draw_idle_id == -1 || !imd->draw_idle_high))
 		{
+		if (imd->draw_idle_id != -1) gtk_idle_remove(imd->draw_idle_id);
 		imd->draw_idle_id = gtk_idle_add_priority(G_PRIORITY_HIGH_IDLE, image_queue_draw_idle_cb, imd);
+		imd->draw_idle_high = TRUE;
 		}
 }
 
-static void image_queue_clear(ImageWindow *imd)
+static void image_queue_list_free(GList *list)
 {
 	GList *work;
 
-	work = imd->draw_queue;
-	while(work)
+	work = list;
+	while (work)
 		{
 		QueueData *qd;
 
@@ -764,9 +727,17 @@ static void image_queue_clear(ImageWindow *imd)
 		g_free(qd);
 		}
 
-	g_list_free(imd->draw_queue);
+	g_list_free(list);
+}
+
+static void image_queue_clear(ImageWindow *imd)
+{
+	image_queue_list_free(imd->draw_queue);
 	imd->draw_queue = NULL;
 
+	image_queue_list_free(imd->draw_queue_2pass);
+	imd->draw_queue_2pass = NULL;
+
 	if (imd->draw_idle_id != -1) gtk_idle_remove(imd->draw_idle_id);
 	imd->draw_idle_id = -1;
 }
@@ -808,10 +779,6 @@ static gint image_size_top_window(ImageWindow *imd, gint w, gint h)
 	gdk_window_get_size(imd->top_window->window, &ww, &wh);
 	if (w == ww && h == wh) return FALSE;
 
-#ifdef IMAGE_DEBUG_ENABLE
-	printf("auto sized to %d x %d (was %d x %d)\n", w, h, ww, wh);
-#endif
-
 	gdk_window_resize(imd->top_window->window, w, h);
 
 	return TRUE;
@@ -1006,10 +973,6 @@ static gint image_size_clamp(ImageWindow *imd)
 		imd->y_offset = 0;
 		}
 
-#ifdef IMAGE_DEBUG_ENABLE
-	printf("size clamp: %d x %d (off: %d, %d)\n", imd->width, imd->height, imd->x_offset, imd->y_offset);
-#endif
-
 	return (old_vw != imd->vis_width || old_vh != imd->vis_height);
 }
 
@@ -1097,10 +1060,6 @@ static void image_scroll_real(ImageWindow *imd, gint x, gint y)
 	imd->x_scroll += x;
 	imd->y_scroll += y;
 
-#ifdef IMAGE_DEBUG_ENABLE
-	printf("scroll by %d, %d\n", x, y);
-#endif
-
 	image_scroll_clamp(imd);
 	if (imd->x_scroll == old_x && imd->y_scroll == old_y) return;
 
@@ -1264,10 +1223,6 @@ static void image_zoom_sync(ImageWindow *imd, gfloat zoom, gint force, gint blan
 
 static void image_pixbuf_sync(ImageWindow *imd, gfloat zoom, gint blank, gint new)
 {
-#ifdef IMAGE_DEBUG_ENABLE
-	printf("syncing pixbuf\n");
-#endif
-
 	if (!imd->pixbuf)
 		{
 		/* no pixbuf so just clear the window */
@@ -1346,10 +1301,7 @@ static void image_alter_real(ImageWindow *imd, AlterType type)
 
 static void image_read_ahead_cancel(ImageWindow *imd)
 {
-
-#ifdef IMAGE_DEBUG_ENABLE
 	if (debug) printf("read ahead cancelled for :%s\n", imd->read_ahead_path);
-#endif
 
 	image_loader_free(imd->read_ahead_il);
 	imd->read_ahead_il = NULL;
@@ -1365,9 +1317,7 @@ static void image_read_ahead_done_cb(ImageLoader *il, gpointer data)
 {
 	ImageWindow *imd = data;
 
-#ifdef IMAGE_DEBUG_ENABLE
 	if (debug) printf("read ahead done for :%s\n", imd->read_ahead_path);
-#endif
 
 	imd->read_ahead_pixbuf = image_loader_get_pixbuf(imd->read_ahead_il);
 	if (imd->read_ahead_pixbuf)
@@ -1396,9 +1346,7 @@ static void image_read_ahead_start(ImageWindow *imd)
 	/* still loading ?, do later */
 	if (imd->il) return;
 
-#ifdef IMAGE_DEBUG_ENABLE
 	if (debug) printf("read ahead started for :%s\n", imd->read_ahead_path);
-#endif
 
 	imd->read_ahead_il = image_loader_new(imd->read_ahead_path);
 
@@ -1417,9 +1365,7 @@ static void image_read_ahead_set(ImageWindow *imd, const gchar *path)
 
 	imd->read_ahead_path = g_strdup(path);
 
-#ifdef IMAGE_DEBUG_ENABLE
 	if (debug) printf("read ahead set to :%s\n", imd->read_ahead_path);
-#endif
 
 	image_read_ahead_start(imd);
 }
@@ -1445,10 +1391,6 @@ static void image_load_area_cb(ImageLoader *il, guint x, guint y, guint w, guint
 {
 	ImageWindow *imd = data;
 
-#ifdef IMAGE_DEBUG_ENABLE
-	printf("image.c: area ready %d, %d (%d x %d)\n", (gint) x, (gint) y, (gint) w, (gint) h);
-#endif
-
 	if (!imd->pixbuf) image_load_pixbuf_ready(imd);
 
 	if (imd->scale != 1.0)
@@ -1471,9 +1413,6 @@ static void image_load_area_cb(ImageLoader *il, guint x, guint y, guint w, guint
 			h += 2;
 			}
 
-#ifdef IMAGE_DEBUG_ENABLE
-		if (debug) printf("image.c: area ready adjusted %d, %d (%d x %d)\n", (gint) x, (gint) y, (gint) w, (gint) h);
-#endif
 		}
 
 	image_queue(imd, (gint) x, (gint) y, (gint) w, (gint) h, FALSE, TILE_RENDER_AREA, TRUE);
@@ -1483,9 +1422,7 @@ static void image_load_done_cb(ImageLoader *il, gpointer data)
 {
 	ImageWindow *imd = data;
 
-#ifdef IMAGE_DEBUG_ENABLE
 	if (debug) printf ("image done\n");
-#endif
 
 	image_loader_free(imd->il);
 	imd->il = NULL;
@@ -1500,10 +1437,7 @@ static void image_load_done_cb(ImageLoader *il, gpointer data)
 
 static void image_load_error_cb(ImageLoader *il, gpointer data)
 {
-
-#ifdef IMAGE_DEBUG_ENABLE
 	if (debug) printf ("image error\n");
-#endif
 
 	/* even on error handle it like it was done,
 	 * since we have a pixbuf with _something_ */
@@ -1511,6 +1445,20 @@ static void image_load_error_cb(ImageLoader *il, gpointer data)
 	image_load_done_cb(il, data);
 }
 
+#ifdef IMAGE_THROTTLE_LARGER_IMAGES
+static void image_load_buffer_throttle(ImageLoader *il)
+{
+	if (!il || il->bytes_total < 1024 * 1024) return;
+
+	/* Larger image files usually have larger chunks of data per pixel...
+	 * So increase the buffer read size so that the rendering chunks called
+	 * are also larger.
+	 */
+
+	image_loader_set_buffer_size(il, IMAGE_LOAD_BUFFER_COUNT * 8);
+}
+#endif
+
 /* this read ahead is located here merely for the callbacks, above */
 
 static gint image_read_ahead_check(ImageWindow *imd)
@@ -1533,6 +1481,11 @@ static gint image_read_ahead_check(ImageWindow *imd)
 		image_loader_set_area_ready_func(imd->il, image_load_area_cb, imd);
 		image_loader_set_error_func(imd->il, image_load_error_cb, imd);
 		image_loader_set_buffer_size(imd->il, IMAGE_LOAD_BUFFER_COUNT);
+
+#ifdef IMAGE_THROTTLE_LARGER_IMAGES
+		image_load_buffer_throttle(imd->il);
+#endif
+
 		/* do this one directly (probably should add a set func) */
 		imd->il->func_done = image_load_done_cb;
 
@@ -1557,19 +1510,13 @@ static gint image_read_ahead_check(ImageWindow *imd)
 
 static gint image_load_begin(ImageWindow *imd, const gchar *path)
 {
-
-#ifdef IMAGE_DEBUG_ENABLE
 	if (debug) printf ("image begin \n");
-#endif
 
 	if (imd->il) return FALSE;
 
 	if (image_read_ahead_check(imd))
 		{
-
-#ifdef IMAGE_DEBUG_ENABLE
 		if (debug) printf("from read ahead buffer: %s\n", imd->image_path);
-#endif
 
 		image_pixbuf_sync(imd, imd->zoom, FALSE, TRUE);
 		return TRUE;
@@ -1583,16 +1530,17 @@ static gint image_load_begin(ImageWindow *imd, const gchar *path)
 
 	if (!image_loader_start(imd->il, image_load_done_cb, imd))
 		{
-
-#ifdef IMAGE_DEBUG_ENABLE
 		if (debug) printf("image start error\n");
-#endif
 
 		image_loader_free(imd->il);
 		imd->il = NULL;
 		return FALSE;
 		}
 
+#ifdef IMAGE_THROTTLE_LARGER_IMAGES
+	image_load_buffer_throttle(imd->il);
+#endif
+
 	if (!imd->pixbuf) image_load_pixbuf_ready(imd);
 
 	return TRUE;
@@ -1602,9 +1550,7 @@ static void image_reset(ImageWindow *imd)
 {
 	/* stops anything currently being done */
 
-#ifdef IMAGE_DEBUG_ENABLE
 	if (debug) printf("image reset\n");
-#endif
 
 	image_loader_free(imd->il);
 	imd->il = NULL;
@@ -1686,13 +1632,13 @@ static void image_change_real(ImageWindow *imd, const gchar *path,
  *-------------------------------------------------------------------
  */
 
-static void image_expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data)
+static gint image_expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data)
 {
 	gint x, y;
 
 	ImageWindow *imd = data;
 
-	if (!imd->pixbuf) return;
+	if (!imd->pixbuf) return FALSE;
 
 	/* border */
 
@@ -1700,27 +1646,18 @@ static void image_expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer d
 	x = MAX(0, (gint)event->area.x - imd->x_offset + imd->x_scroll);
 	y = MAX(0, (gint)event->area.y - imd->y_offset + imd->y_scroll);
 
-#ifdef IMAGE_DEBUG_ENABLE
-	printf("expose event: %d, %d (%d x %d) - [%d x %d (%d x %d)]\n",
-		(gint)event->area.x, (gint)event->area.y,
-		(gint)event->area.width, (gint)event->area.height,
-		x, y, MIN((gint)event->area.width, imd->width - x), MIN((gint)event->area.height, imd->height - y));
-#endif
-
 	image_queue(imd, x, y,
 		    MIN((gint)event->area.width, imd->width - x),
 		    MIN((gint)event->area.height, imd->height - y),
 		    FALSE, TILE_RENDER_ALL, FALSE);
+
+	return TRUE;
 }
 
 static gint image_size_cb(GtkWidget *widget, GtkAllocation *allocation, gpointer data)
 {
 	ImageWindow *imd = data;
 
-#ifdef IMAGE_DEBUG_ENABLE
-	printf("Image window size event: %d x %d\n", allocation->width, allocation->height);
-#endif
-
 	image_size_sync(imd, allocation->width, allocation->height);
 
 	return FALSE;
@@ -1752,7 +1689,15 @@ static gint image_focus_expose(GtkWidget *widget, GdkEventExpose *event, gpointe
 	if (!GTK_WIDGET_HAS_FOCUS(widget)) return FALSE;
 
 	image_focus_paint(imd);
-	return FALSE;
+	return TRUE;
+}
+
+static void image_focus_draw(GtkWidget *widget, GdkRectangle *area, gpointer data)
+{
+	ImageWindow *imd = data;
+
+	if (!GTK_WIDGET_HAS_FOCUS(widget)) return;
+	image_focus_paint(imd);
 }
 
 static gint image_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, gpointer data)
@@ -1781,11 +1726,11 @@ static gint image_focus_out_cb(GtkWidget *widget, GdkEventFocus *event, gpointer
  *-------------------------------------------------------------------
  */
 
-static void image_mouse_motion_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint image_mouse_motion_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	ImageWindow *imd = data;
 
-	if (!imd->in_drag || !gdk_pointer_is_grabbed()) return;
+	if (!imd->in_drag || !gdk_pointer_is_grabbed()) return FALSE;
 
 	if (imd->drag_moved < IMAGE_DRAG_SCROLL_THRESHHOLD)
 		{
@@ -1801,9 +1746,11 @@ static void image_mouse_motion_cb(GtkWidget *widget, GdkEventButton *bevent, gpo
 
 	imd->drag_last_x = bevent->x;
 	imd->drag_last_y = bevent->y;
+
+	return FALSE;
 }
 
-static void image_mouse_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint image_mouse_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	ImageWindow *imd = data;
 
@@ -1832,9 +1779,11 @@ static void image_mouse_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpoi
 		}
 
 	gtk_widget_grab_focus(imd->widget);
+
+	return FALSE;
 }
 
-static void image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	ImageWindow *imd = data;
 
@@ -1851,6 +1800,8 @@ static void image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 		}
 
 	imd->in_drag = FALSE;
+
+	return FALSE;
 }
 
 static void image_mouse_drag_cb(GtkWidget *widget, GdkDragContext *context, gpointer data)
@@ -2241,6 +2192,54 @@ void image_top_window_set_sync(ImageWindow *imd, gint allow_sync)
 	imd->top_window_sync = allow_sync;
 }
 
+/* background colors */
+
+void image_background_set_black(ImageWindow *imd, gint black)
+{
+	GtkStyle *style;
+
+	if (!imd) return;
+
+	style = gtk_style_copy(gtk_widget_get_style(imd->widget));
+	gtk_style_ref(style);
+
+	if (black)
+		{
+		style->bg[GTK_STATE_NORMAL] = style->black;
+		}
+
+	gtk_widget_set_style(imd->image, style);
+	gtk_style_unref(style);
+
+	if (GTK_WIDGET_VISIBLE(imd->widget)) image_border_clear(imd);
+}
+
+void image_background_set_color(ImageWindow *imd, GdkColor *color)
+{
+	GtkStyle *style;
+
+	if (!imd) return;
+
+	style = gtk_style_copy(gtk_widget_get_style(imd->widget));
+	gtk_style_ref(style);
+
+	if (color)
+		{
+		GdkColor *slot;
+
+		slot = &style->bg[GTK_STATE_NORMAL];
+
+		slot->red = color->red;
+		slot->green = color->green;
+		slot->blue = color->blue;
+		}
+
+	gtk_widget_set_style(imd->image, style);
+	gtk_style_unref(style);
+
+	if (GTK_WIDGET_VISIBLE(imd->widget)) image_border_clear(imd);
+}
+
 /* wallpaper util */
 
 void image_to_root_window(ImageWindow *imd, gint scaled)
@@ -2356,7 +2355,7 @@ ImageWindow *image_new(gint frame)
 				   GTK_SIGNAL_FUNC(image_focus_out_cb), imd);
 
 		gtk_signal_connect_after(GTK_OBJECT(imd->widget), "draw",
-					 GTK_SIGNAL_FUNC(image_focus_expose), imd);
+					 GTK_SIGNAL_FUNC(image_focus_draw), imd);
 		gtk_signal_connect_after(GTK_OBJECT(imd->widget), "expose_event",
 					 GTK_SIGNAL_FUNC(image_focus_expose), imd);
 		}
