@@ -59,6 +59,10 @@ void image_prebuffer_set(ImageWindow *imd, const gchar *path);
 /* allow top window to be resized ? */
 void image_top_window_set_sync(ImageWindow *imd, gint allow_sync);
 
+/* background of image */
+void image_background_set_black(ImageWindow *imd, gint black);
+void image_background_set_color(ImageWindow *imd, GdkColor *color);
+
 /* wallpaper util */
 void image_to_root_window(ImageWindow *imd, gint scaled);
 
