@@ -17,6 +17,7 @@
 #include "dnd.h"
 #include "fullscreen.h"
 #include "image.h"
+#include "info.h"
 #include "menu.h"
 #include "slideshow.h"
 #include "utilops.h"
@@ -37,6 +38,9 @@ struct _ViewWindow
 };
 
 
+static GList *view_window_list = NULL;
+
+
 static GtkWidget *view_popup_menu(ViewWindow *vw);
 static void view_fullscreen_toggle(ViewWindow *vw, gint force_off);
 
@@ -484,7 +488,8 @@ static void view_fullscreen_toggle(ViewWindow *vw, gint force_off)
 		vw->fs = fullscreen_start(vw->window, vw->imd, view_fullscreen_stop_func, vw);
 
 		view_image_set_buttons(vw, vw->fs->imd);
-		gtk_signal_connect(GTK_OBJECT(vw->fs->window), "key_press_event", GTK_SIGNAL_FUNC(view_window_key_press_cb), vw);
+		gtk_signal_connect(GTK_OBJECT(vw->fs->window), "key_press_event",
+				   GTK_SIGNAL_FUNC(view_window_key_press_cb), vw);
 
 		if (vw->ss) vw->ss->imd = vw->fs->imd;
 		}
@@ -530,6 +535,8 @@ static void view_slideshow_stop(ViewWindow *vw)
 
 static void view_window_close(ViewWindow *vw)
 {
+	view_window_list = g_list_remove(view_window_list, vw);
+
 	view_slideshow_stop(vw);
 	view_fullscreen_toggle(vw, TRUE);
 	gtk_widget_destroy(vw->window);
@@ -565,6 +572,9 @@ static void real_view_window_new(const gchar *path, CollectionData *cd, CollectI
 	window_set_icon(vw->window, (const gchar **)view_xpm, NULL);
 
 	vw->imd = image_new(FALSE);
+
+	if (black_window_background) image_background_set_black(vw->imd, TRUE);
+
 	image_attach_window(vw->imd, vw->window, NULL, " - GQview", TRUE);
 
 	image_top_window_set_sync(vw->imd, TRUE);
@@ -576,8 +586,10 @@ static void real_view_window_new(const gchar *path, CollectionData *cd, CollectI
 
 	view_image_set_buttons(vw, vw->imd);
 
-	gtk_signal_connect(GTK_OBJECT(vw->window), "delete_event", (GtkSignalFunc) view_window_delete_cb, vw);
-	gtk_signal_connect(GTK_OBJECT(vw->window), "key_press_event", GTK_SIGNAL_FUNC(view_window_key_press_cb), vw);
+	gtk_signal_connect(GTK_OBJECT(vw->window), "delete_event",
+			   GTK_SIGNAL_FUNC(view_window_delete_cb), vw);
+	gtk_signal_connect(GTK_OBJECT(vw->window), "key_press_event",
+			   GTK_SIGNAL_FUNC(view_window_key_press_cb), vw);
 
 	if (cd && info)
 		{
@@ -622,6 +634,8 @@ static void real_view_window_new(const gchar *path, CollectionData *cd, CollectI
 	gtk_widget_set_usize(vw->imd->eventbox, w, h);
 
 	gtk_widget_show(vw->window);
+
+	view_window_list = g_list_append(view_window_list, vw);
 }
 
 void view_window_new(const gchar *path)
@@ -634,6 +648,26 @@ void view_window_new_from_collection(CollectionData *cd, CollectInfo *info)
 	real_view_window_new(NULL, cd, info);
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ * public
+ *-----------------------------------------------------------------------------
+ */
+
+void view_window_colors_update(void)
+{
+	GList *work;
+
+	work = view_window_list;
+	while (work)
+		{
+		ViewWindow *vw = work->data;
+		work = work->next;
+
+		image_background_set_black(vw->imd, black_window_background);
+		}
+}
+
 /*
  *-----------------------------------------------------------------------------
  * view window menu routines and callbacks
@@ -683,6 +717,13 @@ static void view_alter_cb(GtkWidget *widget, gpointer data)
 	image_alter(vw->imd, type);
 }
 
+static void view_info_cb(GtkWidget *widget, gpointer data)
+{
+	ViewWindow *vw = data;
+
+	info_window_new(image_get_path(vw->imd), NULL);
+}
+
 static void view_wallpaper_cb(GtkWidget *widget, gpointer data)
 {
 	ViewWindow *vw = data;
@@ -795,25 +836,27 @@ static GtkWidget *view_popup_menu(ViewWindow *vw)
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add(menu, _("Zoom in"), view_zoom_in_cb, vw);
-	menu_item_add(menu, _("Zoom out"), view_zoom_out_cb, vw);
-	menu_item_add(menu, _("Zoom 1:1"), view_zoom_1_1_cb, vw);
-	menu_item_add(menu, _("Fit image to window"), view_zoom_fit_cb, vw);
+	menu_item_add(menu, _("Zoom in"), GTK_SIGNAL_FUNC(view_zoom_in_cb), vw);
+	menu_item_add(menu, _("Zoom out"), GTK_SIGNAL_FUNC(view_zoom_out_cb), vw);
+	menu_item_add(menu, _("Zoom 1:1"), GTK_SIGNAL_FUNC(view_zoom_1_1_cb), vw);
+	menu_item_add(menu, _("Fit image to window"), GTK_SIGNAL_FUNC(view_zoom_fit_cb), vw);
 	menu_item_add_divider(menu);
 
-	item = submenu_add_edit(menu, NULL, view_edit_cb, vw);
+	item = submenu_add_edit(menu, NULL, GTK_SIGNAL_FUNC(view_edit_cb), vw);
 	menu_item_add_divider(item);
-	menu_item_add(item, _("Set as wallpaper"), view_wallpaper_cb, vw);
+	menu_item_add(item, _("Set as wallpaper"), GTK_SIGNAL_FUNC(view_wallpaper_cb), vw);
 
-	submenu_add_alter(menu, view_alter_cb, vw);
+	submenu_add_alter(menu, GTK_SIGNAL_FUNC(view_alter_cb), vw);
 
-	menu_item_add(menu, _("View in new window"), view_new_window_cb, vw);
+	menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(view_info_cb), vw);
+
+	menu_item_add(menu, _("View in new window"), GTK_SIGNAL_FUNC(view_new_window_cb), vw);
 
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Copy..."), view_copy_cb, vw);
-	menu_item_add(menu, _("Move..."), view_move_cb, vw);
-	menu_item_add(menu, _("Rename..."), view_rename_cb, vw);
-	menu_item_add(menu, _("Delete..."), view_delete_cb, vw);
+	menu_item_add(menu, _("Copy..."), GTK_SIGNAL_FUNC(view_copy_cb), vw);
+	menu_item_add(menu, _("Move..."), GTK_SIGNAL_FUNC(view_move_cb), vw);
+	menu_item_add(menu, _("Rename..."), GTK_SIGNAL_FUNC(view_rename_cb), vw);
+	menu_item_add(menu, _("Delete..."), GTK_SIGNAL_FUNC(view_delete_cb), vw);
 
 	menu_item_add_divider(menu);
 
@@ -822,32 +865,34 @@ static GtkWidget *view_popup_menu(ViewWindow *vw)
 		menu_item_add(menu, _("Stop slideshow"), view_slideshow_stop_cb, vw);
 		if (slideshow_paused(vw->ss))
 			{
-			item = menu_item_add(menu, _("Continue slideshow"), view_slideshow_pause_cb, vw);
+			item = menu_item_add(menu, _("Continue slideshow"),
+					     GTK_SIGNAL_FUNC(view_slideshow_pause_cb), vw);
 			}
 		else
 			{
-			item = menu_item_add(menu, _("Pause slideshow"), view_slideshow_pause_cb, vw);
+			item = menu_item_add(menu, _("Pause slideshow"),
+					     GTK_SIGNAL_FUNC(view_slideshow_pause_cb), vw);
 			}
 		}
 	else
 		{
-		item = menu_item_add(menu, _("Start slideshow"), view_slideshow_start_cb, vw);
+		item = menu_item_add(menu, _("Start slideshow"), GTK_SIGNAL_FUNC(view_slideshow_start_cb), vw);
 		gtk_widget_set_sensitive(item, view_window_contains_collection(vw));
-		item = menu_item_add(menu, _("Pause slideshow"), view_slideshow_pause_cb, vw);
+		item = menu_item_add(menu, _("Pause slideshow"), GTK_SIGNAL_FUNC(view_slideshow_pause_cb), vw);
 		gtk_widget_set_sensitive(item, FALSE);
 		}
 
 	if (vw->fs)
 		{
-		menu_item_add(menu, _("Exit full screen"), view_fullscreen_cb, vw);
+		menu_item_add(menu, _("Exit full screen"), GTK_SIGNAL_FUNC(view_fullscreen_cb), vw);
 		}
 	else
 		{
-		menu_item_add(menu, _("Full screen"), view_fullscreen_cb, vw);
+		menu_item_add(menu, _("Full screen"), GTK_SIGNAL_FUNC(view_fullscreen_cb), vw);
 		}
 
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Close window"), view_close_cb, vw);
+	menu_item_add(menu, _("Close window"), GTK_SIGNAL_FUNC(view_close_cb), vw);
 
 	return menu;
 }
