@@ -25,17 +25,20 @@
 #include "ui_menu.h"
 #include "ui_tabcomp.h"
 
+#include "icons/tools.xpm"
 
 #define MAINWINDOW_DEF_WIDTH 500
 #define MAINWINDOW_DEF_HEIGHT 400
 
 #define MAIN_WINDOW_DIV_HPOS -1
-#define MAIN_WINDOW_DIV_VPOS 100
+#define MAIN_WINDOW_DIV_VPOS 200
 
 #define TOOLWINDOW_DEF_WIDTH 224
 #define TOOLWINDOW_DEF_HEIGHT 450
 #define PROGRESS_WIDTH 150
 
+#define PANE_DIVIDER_SIZE 10
+
 
 GList *layout_window_list = NULL;
 
@@ -157,7 +160,6 @@ static GtkWidget *layout_tool_setup(LayoutWindow *lw)
 {
 	GtkWidget *box;
 	GtkWidget *menu_bar;
-	GtkWidget *button_bar;
 	GtkWidget *tabcomp;
 	GtkWidget *popwin;
 
@@ -167,9 +169,9 @@ static GtkWidget *layout_tool_setup(LayoutWindow *lw)
 	gtk_box_pack_start(GTK_BOX(box), menu_bar, FALSE, FALSE, 0);
 	gtk_widget_show(menu_bar);
 
-	button_bar = layout_button_bar(lw);
-	gtk_box_pack_start(GTK_BOX(box), button_bar, FALSE, FALSE, 0);
-	gtk_widget_show(button_bar);
+	lw->toolbar = layout_button_bar(lw);
+	gtk_box_pack_start(GTK_BOX(box), lw->toolbar, FALSE, FALSE, 0);
+	if (!lw->toolbar_hidden) gtk_widget_show(lw->toolbar);
 
 	tabcomp = tab_completion_new_with_history(&lw->path_entry, NULL, "path_list", 32,
 						  layout_path_entry_cb, lw);
@@ -244,13 +246,14 @@ static void layout_sort_button_press_cb(GtkWidget *widget, gpointer data)
 	LayoutWindow *lw = data;
 	GtkWidget *menu;
 
-	menu = submenu_add_sort(NULL, layout_sort_menu_cb, lw, FALSE, FALSE, TRUE, lw->sort_method);
+	menu = submenu_add_sort(NULL, GTK_SIGNAL_FUNC(layout_sort_menu_cb), lw, FALSE, FALSE, TRUE, lw->sort_method);
 
         /* ascending option */
 	menu_item_add_divider(menu);
-	menu_item_add_check(menu, _("Ascending"), lw->sort_ascend, layout_sort_menu_ascend_cb, lw);
+	menu_item_add_check(menu, _("Ascending"), lw->sort_ascend, GTK_SIGNAL_FUNC(layout_sort_menu_ascend_cb), lw);
 
-	gtk_signal_connect(GTK_OBJECT(menu), "selection_done", layout_sort_menu_hide_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(menu), "selection_done",
+			   GTK_SIGNAL_FUNC(layout_sort_menu_hide_cb), NULL);
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gdk_time_get());
 }
 
@@ -259,7 +262,8 @@ static GtkWidget *layout_sort_button(LayoutWindow *lw)
 	GtkWidget *button;
 
 	button = gtk_button_new_with_label(sort_type_get_text(lw->sort_method));
-        gtk_signal_connect(GTK_OBJECT(button), "clicked", layout_sort_button_press_cb, lw);
+        gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(layout_sort_button_press_cb), lw);
         gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 
         return button;
@@ -274,6 +278,7 @@ static GtkWidget *layout_sort_button(LayoutWindow *lw)
 void layout_status_update_progress(LayoutWindow *lw, gfloat val, const gchar *text)
 {
 	if (!layout_valid(&lw)) return;
+	if (!lw->info_progress_bar) return;
 
 	gtk_progress_bar_update(GTK_PROGRESS_BAR(lw->info_progress_bar), val);
 	gtk_progress_set_format_string(GTK_PROGRESS(lw->info_progress_bar), text ? text : "");
@@ -355,7 +360,14 @@ void layout_status_update_image(LayoutWindow *lw)
 
 	if (lw->image->unknown)
 		{
-		text = g_strdup_printf(_("( ? x ? ) %s bytes"), b);
+		if (image_get_path(lw->image) && access(image_get_path(lw->image), R_OK) < 0)
+			{
+			text = g_strdup_printf(_("(no read permission) %s bytes"), b);
+			}
+		else
+			{
+			text = g_strdup_printf(_("( ? x ? ) %s bytes"), b);
+			}
 		}
 	else
 		{
@@ -381,9 +393,9 @@ static GtkWidget *layout_status_label(gchar *text, GtkWidget *box, gint start, g
 	GtkWidget *label;
 	GtkWidget *frame;
 
-	frame = gtk_frame_new (NULL);
+	frame = gtk_frame_new(NULL);
 	if (size) gtk_widget_set_usize (frame, size, -1);
-	gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_IN);
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
 	if (start)
 		{
 		gtk_box_pack_start(GTK_BOX(box), frame, expand, expand, 0);
@@ -395,33 +407,63 @@ static GtkWidget *layout_status_label(gchar *text, GtkWidget *box, gint start, g
 	gtk_widget_show(frame);
 
 	label = gtk_label_new(text ? text : "");
-	gtk_container_add (GTK_CONTAINER (frame), label);
+	gtk_container_add(GTK_CONTAINER(frame), label);
 	gtk_widget_show(label);
 
 	return label;
 }
 
-static void layout_status_setup(LayoutWindow *lw)
+static void layout_status_setup(LayoutWindow *lw, GtkWidget *box, gint small_format)
 {
+	GtkWidget *hbox;
+
 	if (lw->info_box) return;
 
-	lw->info_box = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(lw->main_box), lw->info_box, FALSE, FALSE, 0);
+	if (small_format)
+		{
+		lw->info_box = gtk_vbox_new(FALSE, 0);
+		}
+	else
+		{
+		lw->info_box = gtk_hbox_new(FALSE, 0);
+		}
+	gtk_box_pack_end(GTK_BOX(box), lw->info_box, FALSE, FALSE, 0);
 	gtk_widget_show(lw->info_box);
 
+	if (small_format)
+		{
+		hbox = gtk_hbox_new(FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(lw->info_box), hbox, FALSE, FALSE, 0);
+		gtk_widget_show(hbox);
+		}
+	else
+		{
+		hbox = lw->info_box;
+		}
 	lw->info_progress_bar = gtk_progress_bar_new();
 	gtk_progress_set_show_text(GTK_PROGRESS(lw->info_progress_bar), TRUE);
 	gtk_widget_set_usize(lw->info_progress_bar, PROGRESS_WIDTH, -1);
-	gtk_box_pack_start(GTK_BOX(lw->info_box), lw->info_progress_bar, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), lw->info_progress_bar, FALSE, FALSE, 0);
 	gtk_widget_show(lw->info_progress_bar);
 
 	lw->info_sort = layout_sort_button(lw);
-	gtk_box_pack_start(GTK_BOX(lw->info_box), lw->info_sort, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), lw->info_sort, FALSE, FALSE, 0);
 	gtk_widget_show(lw->info_sort);
 
-	lw->info_status = layout_status_label(NULL, lw->info_box, TRUE, 0, TRUE);
-	lw->info_details = layout_status_label(NULL, lw->info_box, TRUE, 0, TRUE);
-	lw->info_zoom = layout_status_label(NULL, lw->info_box, FALSE, 48, FALSE);
+	lw->info_status = layout_status_label(NULL, lw->info_box, TRUE, 0, (!small_format));
+
+	if (small_format)
+		{
+		hbox = gtk_hbox_new(FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(lw->info_box), hbox, FALSE, FALSE, 0);
+		gtk_widget_show(hbox);
+		}
+	else
+		{
+		hbox = lw->info_box;
+		}
+	lw->info_details = layout_status_label(NULL, hbox, TRUE, 0, TRUE);
+	lw->info_zoom = layout_status_label(NULL, hbox, FALSE, 48, FALSE);
 }
 
 /*
@@ -744,8 +786,23 @@ gint layout_geometry_get_dividers(LayoutWindow *lw, gint *h, gint *v)
 {
 	if (!layout_valid(&lw)) return FALSE;
 
-	*h = GTK_PANED(lw->h_pane)->child1->allocation.width;
-	*v = GTK_PANED(lw->v_pane)->child1->allocation.height;
+	if (lw->h_pane)
+		{
+		*h = GTK_PANED(lw->h_pane)->child1->allocation.width;
+		}
+	else if (h != &lw->div_h)
+		{
+		*h = lw->div_h;
+		}
+
+	if (lw->v_pane)
+		{
+		*v = GTK_PANED(lw->v_pane)->child1->allocation.height;
+		}
+	else if (v != &lw->div_v)
+		{
+		*v = lw->div_v;
+		}
 
 	return TRUE;
 }
@@ -774,7 +831,7 @@ gint layout_views_get(LayoutWindow *lw, gint *tree, gint *icons)
 
 /*
  *-----------------------------------------------------------------------------
- * glue (layout arrangement)
+ * location utils
  *-----------------------------------------------------------------------------
  */
 
@@ -805,9 +862,9 @@ static LayoutLocation layout_grid_compass(LayoutWindow *lw)
 	return lw->image_location;
 }
 
-static void layout_grid_compute_2(LayoutLocation l1, LayoutLocation l2,
-				  GtkWidget *s1, GtkWidget *s2,
-				  GtkWidget **d1, GtkWidget **d2)
+static void layout_location_compute(LayoutLocation l1, LayoutLocation l2,
+				    GtkWidget *s1, GtkWidget *s2,
+				    GtkWidget **d1, GtkWidget **d2)
 {
 	LayoutLocation l;
 
@@ -826,6 +883,182 @@ static void layout_grid_compute_2(LayoutLocation l1, LayoutLocation l2,
 		}
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ * tools window (for floating/hidden)
+ *-----------------------------------------------------------------------------
+ */
+
+gint layout_geometry_get_tools(LayoutWindow *lw, gint *x, gint *y, gint *w, gint *h, gint *divider_pos)
+{
+	if (!layout_valid(&lw)) return FALSE;
+
+	if (!lw->tools || !GTK_WIDGET_VISIBLE(lw->tools))
+		{
+		/* use the stored values (sort of breaks success return value) */
+
+		*divider_pos = lw->div_float;
+
+		return FALSE;
+		}
+
+	gdk_window_get_root_origin(lw->tools->window, x, y);
+	gdk_window_get_size(lw->tools->window, w, h);
+
+	if (GTK_IS_VPANED(lw->tools_pane))
+		{
+		*divider_pos = GTK_PANED(lw->tools_pane)->child1->allocation.height;
+		}
+	else
+		{
+		*divider_pos = GTK_PANED(lw->tools_pane)->child1->allocation.width;
+		}
+
+	return TRUE;
+}
+
+static void layout_tools_geometry_sync(LayoutWindow *lw)
+{
+	layout_geometry_get_tools(lw, &float_window_x, &float_window_x,
+				  &float_window_w, &float_window_h, &lw->div_float);
+}
+
+static void layout_tools_hide(LayoutWindow *lw, gint hide)
+{
+	if (!lw->tools) return;
+
+	if (hide)
+		{
+		if (GTK_WIDGET_VISIBLE(lw->tools))
+			{
+			layout_tools_geometry_sync(lw);
+			gtk_widget_hide(lw->tools);
+			}
+		}
+	else
+		{
+		if (!GTK_WIDGET_VISIBLE(lw->tools)) gtk_widget_show(lw->tools);
+		}
+
+	lw->tools_hidden = hide;
+}
+
+static gint layout_tools_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	layout_tools_float_toggle(lw);
+
+	return TRUE;
+}
+
+static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *files)
+{
+	GtkWidget *vbox;
+	GtkWidget *w1, *w2;
+	gint vertical;
+	gint new_window = FALSE;
+
+	vertical = (layout_location_single(lw->image_location) && !layout_location_vertical(lw->image_location)) ||
+		   (!layout_location_single(lw->image_location) && layout_location_vertical(layout_grid_compass(lw)));
+#if 0
+	layout_location_compute(lw->dir_location, lw->file_location,
+				tools, files, &w1, &w2);
+#endif
+	/* for now, tools/dir are always first in order */
+	w1 = tools;
+	w2 = files;
+
+	if (!lw->tools)
+		{
+		lw->tools = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_signal_connect(GTK_OBJECT(lw->tools), "delete_event",
+				   GTK_SIGNAL_FUNC(layout_tools_delete_cb), lw);
+	        gtk_window_set_policy(GTK_WINDOW(lw->tools), TRUE, TRUE, FALSE);
+        	gtk_window_set_title(GTK_WINDOW(lw->tools), _("GQview Tools"));
+        	gtk_window_set_wmclass(GTK_WINDOW(lw->tools), "tools", "GQview");
+        	gtk_container_border_width(GTK_CONTAINER(lw->tools), 0);
+	        gtk_window_add_accel_group(GTK_WINDOW(lw->tools), lw->accel_grp);
+
+		window_set_icon(lw->tools, (const gchar **)tools_xpm, NULL);
+
+		new_window = TRUE;
+		}
+	else
+		{
+		layout_tools_geometry_sync(lw);
+		/* dump the contents */
+		gtk_widget_destroy(GTK_BIN(lw->tools)->child);
+		}
+
+	vbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(lw->tools), vbox);
+	gtk_widget_show(vbox);
+
+	layout_status_setup(lw, vbox, TRUE);
+
+	if (vertical)
+		{
+		lw->tools_pane = gtk_vpaned_new();
+		}
+	else
+		{
+		lw->tools_pane = gtk_hpaned_new();
+		}
+	gtk_paned_handle_size(GTK_PANED(lw->tools_pane), PANE_DIVIDER_SIZE);
+	gtk_paned_gutter_size(GTK_PANED(lw->tools_pane), PANE_DIVIDER_SIZE);
+	gtk_box_pack_start(GTK_BOX(vbox), lw->tools_pane, TRUE, TRUE, 0);
+	gtk_widget_show(lw->tools_pane);
+
+	gtk_paned_pack1(GTK_PANED(lw->tools_pane), w1, FALSE, TRUE);
+	gtk_paned_pack2(GTK_PANED(lw->tools_pane), w2, TRUE, TRUE);
+
+	gtk_widget_show(tools);
+	gtk_widget_show(files);
+
+	if (new_window)
+		{
+		if (save_window_positions)
+			{
+			gtk_window_set_default_size(GTK_WINDOW(lw->tools), float_window_w, float_window_h);
+			gtk_widget_set_uposition(lw->tools, float_window_x, float_window_y);
+        	        }
+		else
+			{
+			if (vertical)
+				{
+				gtk_window_set_default_size(GTK_WINDOW(lw->tools),
+							    TOOLWINDOW_DEF_WIDTH, TOOLWINDOW_DEF_HEIGHT);
+				}
+			else
+				{
+				gtk_window_set_default_size(GTK_WINDOW(lw->tools),
+							    TOOLWINDOW_DEF_HEIGHT, TOOLWINDOW_DEF_WIDTH);
+				}
+			}
+		}
+
+	if (!save_window_positions)
+		{
+		if (vertical)
+			{
+			lw->div_float = MAIN_WINDOW_DIV_VPOS;
+			}
+		else
+			{
+			lw->div_float = MAIN_WINDOW_DIV_HPOS;
+			}
+		}
+
+	gtk_paned_set_position(GTK_PANED(lw->tools_pane), lw->div_float);
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ * glue (layout arrangement)
+ *-----------------------------------------------------------------------------
+ */
+
 static void layout_grid_compute(LayoutWindow *lw,
 				GtkWidget *image, GtkWidget *tools, GtkWidget *files,
 				GtkWidget **w1, GtkWidget **w2, GtkWidget **w3)
@@ -837,12 +1070,12 @@ static void layout_grid_compute(LayoutWindow *lw,
 		if (layout_location_first(lw->dir_location))
 			{
 			*w1 = tools;
-			layout_grid_compute_2(lw->file_location, lw->image_location, files, image, w2, w3);
+			layout_location_compute(lw->file_location, lw->image_location, files, image, w2, w3);
 			}
 		else
 			{
 			*w3 = tools;
-			layout_grid_compute_2(lw->file_location, lw->image_location, files, image, w1, w2);
+			layout_location_compute(lw->file_location, lw->image_location, files, image, w1, w2);
 			}
 		}
 	else if (layout_location_single(lw->file_location))
@@ -850,12 +1083,12 @@ static void layout_grid_compute(LayoutWindow *lw,
 		if (layout_location_first(lw->file_location))
 			{
 			*w1 = files;
-			layout_grid_compute_2(lw->dir_location, lw->image_location, tools, image, w2, w3);
+			layout_location_compute(lw->dir_location, lw->image_location, tools, image, w2, w3);
 			}
 		else
 			{
 			*w3 = files;
-			layout_grid_compute_2(lw->dir_location, lw->image_location, tools, image, w1, w2);
+			layout_location_compute(lw->dir_location, lw->image_location, tools, image, w1, w2);
 			}
 		}
 	else
@@ -864,20 +1097,17 @@ static void layout_grid_compute(LayoutWindow *lw,
 		if (layout_location_first(lw->image_location))
 			{
 			*w1 = image;
-			layout_grid_compute_2(lw->file_location, lw->dir_location, files, tools, w2, w3);
+			layout_location_compute(lw->file_location, lw->dir_location, files, tools, w2, w3);
 			}
 		else
 			{
 			*w3 = image;
-			layout_grid_compute_2(lw->file_location, lw->dir_location, files, tools, w1, w2);
+			layout_location_compute(lw->file_location, lw->dir_location, files, tools, w1, w2);
 			}
 		}
 }
 
-
-#define PANE_DIVIDER_SIZE 10
-
-static void layout_grid_setup(LayoutWindow *lw, gint restore, gint h_pos, gint v_pos)
+static void layout_grid_setup(LayoutWindow *lw)
 {
 	gint priority_location;
 	GtkWidget *h;
@@ -888,12 +1118,35 @@ static void layout_grid_setup(LayoutWindow *lw, gint restore, gint h_pos, gint v
 	GtkWidget *tools;
 	GtkWidget *files;
 
+	lw->group_box = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(lw->main_box), lw->group_box, TRUE, TRUE, 0);
+	gtk_widget_show(lw->group_box);
+
 	priority_location = layout_grid_compass(lw);
 
 	image = layout_image_new(lw, NULL);
 	tools = layout_tools_new(lw);
 	files = layout_list_new(lw);
 
+	if (lw->tools_float || lw->tools_hidden)
+		{
+		gtk_box_pack_start(GTK_BOX(lw->group_box), image, TRUE, TRUE, 0);
+		gtk_widget_show(image);
+
+		layout_tools_setup(lw, tools, files);
+
+		return;
+		}
+	else if (lw->tools)
+		{
+		layout_tools_geometry_sync(lw);
+		gtk_widget_destroy(lw->tools);
+		lw->tools = NULL;
+		lw->tools_pane = NULL;
+		}
+
+	layout_status_setup(lw, lw->group_box, FALSE);
+
 	layout_grid_compute(lw, image, tools, files, &w1, &w2, &w3);
 
 	v = lw->v_pane = gtk_vpaned_new();
@@ -913,10 +1166,6 @@ static void layout_grid_setup(LayoutWindow *lw, gint restore, gint h_pos, gint v
 		h = tmp;
 		}
 
-	lw->group_box = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(lw->main_box), lw->group_box, TRUE, TRUE, 0);
-	gtk_widget_show(lw->group_box);
-
 	gtk_box_pack_start(GTK_BOX(lw->group_box), v, TRUE, TRUE, 0);
 
 	if (!layout_location_first(priority_location))
@@ -943,28 +1192,14 @@ static void layout_grid_setup(LayoutWindow *lw, gint restore, gint h_pos, gint v
 	gtk_widget_show(v);
 	gtk_widget_show(h);
 
-	if (restore)
-		{
-		gtk_paned_set_position(GTK_PANED(lw->h_pane), h_pos);
-		gtk_paned_set_position(GTK_PANED(lw->v_pane), v_pos);
-		}
-	else if (save_window_positions)
-		{
-		gtk_paned_set_position(GTK_PANED(lw->h_pane), window_hdivider_pos);
-		gtk_paned_set_position(GTK_PANED(lw->v_pane), window_vdivider_pos);
-		}
-	else
-		{
-		gtk_paned_set_position(GTK_PANED(lw->h_pane), MAIN_WINDOW_DIV_HPOS);
-		gtk_paned_set_position(GTK_PANED(lw->v_pane), MAIN_WINDOW_DIV_VPOS);
-		}
+	gtk_paned_set_position(GTK_PANED(lw->h_pane), lw->div_h);
+	gtk_paned_set_position(GTK_PANED(lw->v_pane), lw->div_v);
 }
 
 void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 {
 	gchar *path;
 	ImageWindow *old_image;
-	gint hpane_pos, vpane_pos;
 
 	if (!layout_valid(&lw)) return;
 
@@ -988,11 +1223,12 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	layout_image_slideshow_stop(lw);
 	layout_image_full_screen_stop(lw);
 
-	path = g_strdup(lw->path);
+	path = lw->path;
 	lw->path = NULL;
 	old_image = lw->image;
 	lw->image = NULL;
-	layout_geometry_get_dividers(lw, &hpane_pos, &vpane_pos);
+
+	layout_geometry_get_dividers(lw, &lw->div_h, &lw->div_v);
 
 	/* clear it all */
 
@@ -1009,6 +1245,7 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	lw->v_pane = NULL;
 
 	lw->menu_fact = NULL;
+	lw->toolbar = NULL;
 	lw->thumb_button = NULL;
 	lw->path_entry = NULL;
 	lw->dir_view = NULL;
@@ -1019,9 +1256,17 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	lw->vfl = NULL;
 	lw->vfi = NULL;
 
+	lw->info_box = NULL;
+	lw->info_progress_bar = NULL;
+	lw->info_sort = NULL;
+	lw->info_status = NULL;
+	lw->info_details = NULL;
+	lw->info_zoom = NULL;
+
 	/* re-fill */
 
-	layout_grid_setup(lw, TRUE, hpane_pos, vpane_pos);
+	layout_grid_setup(lw);
+	layout_tools_hide(lw, lw->tools_hidden);
 
 	layout_list_sync_sort(lw);
 	layout_util_sync(lw);
@@ -1052,6 +1297,114 @@ void layout_styles_update(void)
 		}
 }
 
+void layout_colors_update(void)
+{
+	GList *work;
+
+	work = layout_window_list;
+	while (work)
+		{
+		LayoutWindow *lw = work->data;
+		work = work->next;
+
+		if (lw->image) image_background_set_black(lw->image, black_window_background);
+		}
+}
+
+void layout_tools_float_toggle(LayoutWindow *lw)
+{
+	gint popped;
+
+	if (!lw) return;
+
+	if (!lw->tools_hidden)
+		{
+		popped = !lw->tools_float;
+		}
+	else
+		{
+		popped = TRUE;
+		}
+
+	if (lw->tools_float == popped)
+		{
+		if (popped && lw->tools_hidden)
+			{
+			layout_tools_float_set(lw, popped, FALSE);
+			}
+		}
+	else
+		{
+		if (lw->tools_float)
+			{
+			layout_tools_float_set(lw, FALSE, FALSE);
+			}
+		else
+			{
+			layout_tools_float_set(lw, TRUE, FALSE);
+			}
+		}
+}
+
+void layout_tools_hide_toggle(LayoutWindow *lw)
+{
+	if (!lw) return;
+
+	layout_tools_float_set(lw, lw->tools_float, !lw->tools_hidden);
+}
+
+void layout_tools_float_set(LayoutWindow *lw, gint popped, gint hidden)
+{
+	if (!lw) return;
+
+	if (lw->tools_float == popped && lw->tools_hidden == hidden) return;
+
+	if (lw->tools_float == popped && lw->tools_float && lw->tools)
+		{
+		layout_tools_hide(lw, hidden);
+		return;
+		}
+
+	lw->tools_float = popped;
+	lw->tools_hidden = hidden;
+
+	layout_style_set(lw, -1, NULL);
+}
+
+gint layout_tools_float_get(LayoutWindow *lw, gint *popped, gint *hidden)
+{
+	if (!layout_valid(&lw)) return FALSE;
+
+	*popped = lw->tools_float;
+	*hidden = lw->tools_hidden;
+
+	return TRUE;
+}
+
+void layout_toolbar_toggle(LayoutWindow *lw)
+{
+	if (!layout_valid(&lw)) return;
+	if (!lw->toolbar) return;
+
+	lw->toolbar_hidden = !lw->toolbar_hidden;
+
+	if (lw->toolbar_hidden)
+		{
+		if (GTK_WIDGET_VISIBLE(lw->toolbar)) gtk_widget_hide(lw->toolbar);
+		}
+	else
+		{
+		if (!GTK_WIDGET_VISIBLE(lw->toolbar)) gtk_widget_show(lw->toolbar);
+		}
+}
+
+gint layout_toolbar_hidden(LayoutWindow *lw)
+{
+	if (!layout_valid(&lw)) return TRUE;
+
+	return lw->toolbar_hidden;
+}
+
 /*
  *-----------------------------------------------------------------------------
  * base
@@ -1092,7 +1445,7 @@ static gint layout_delete_cb(GtkWidget *widget, gpointer data)
 	return TRUE;
 }
 
-LayoutWindow *layout_new(const gchar *path)
+LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 {
 	LayoutWindow *lw;
 
@@ -1102,6 +1455,11 @@ LayoutWindow *layout_new(const gchar *path)
 	lw->sort_method = SORT_NAME;
 	lw->sort_ascend = TRUE;
 
+	lw->tools_float = popped;
+	lw->tools_hidden = hidden;
+
+	lw->toolbar_hidden = toolbar_hidden;
+
 	/* default layout */
 
 	layout_config_parse(layout_style, layout_order,
@@ -1109,6 +1467,21 @@ LayoutWindow *layout_new(const gchar *path)
 	lw->tree_view = layout_view_tree;
 	lw->icon_view = layout_view_icons;
 
+	/* divider positions */
+
+	if (save_window_positions)
+		{
+		lw->div_h = window_hdivider_pos;
+		lw->div_v = window_vdivider_pos;
+		lw->div_float = float_window_divider;
+		}
+	else
+		{
+		lw->div_h = MAIN_WINDOW_DIV_HPOS;
+		lw->div_v = MAIN_WINDOW_DIV_VPOS;
+		lw->div_float = MAIN_WINDOW_DIV_VPOS;
+		}
+
 	/* window */
 
 	lw->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -1149,8 +1522,7 @@ LayoutWindow *layout_new(const gchar *path)
 	gtk_container_add(GTK_CONTAINER(lw->window), lw->main_box);
 	gtk_widget_show(lw->main_box);
 
-	layout_status_setup(lw);
-	layout_grid_setup(lw, FALSE, 0, 0);
+	layout_grid_setup(lw);
 
 	layout_util_sync(lw);
 	layout_status_update_all(lw);
@@ -1169,6 +1541,7 @@ LayoutWindow *layout_new(const gchar *path)
 		}
 
 	gtk_widget_show(lw->window);
+	layout_tools_hide(lw, lw->tools_hidden);
 
 	layout_window_list = g_list_append(layout_window_list, lw);
 
@@ -1241,4 +1614,3 @@ void layout_maint_moved(const gchar *source, const gchar *dest, GList *ignore_li
 		}
 }
 
-
