@@ -17,6 +17,7 @@
 #include "fullscreen.h"
 #include "image.h"
 #include "img-view.h"
+#include "info.h"
 #include "layout.h"
 #include "menu.h"
 #include "utilops.h"
@@ -491,6 +492,13 @@ static void li_pop_menu_alter_cb(GtkWidget *widget, gpointer data)
 	image_alter(lw->image, type);
 }
 
+static void li_pop_menu_info_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	info_window_new(layout_image_get_path(lw), NULL);
+}
+
 static void li_pop_menu_new_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
@@ -560,10 +568,9 @@ static void li_pop_menu_full_screen_cb(GtkWidget *widget, gpointer data)
 
 static void li_pop_menu_hide_cb(GtkWidget *widget, gpointer data)
 {
-#if 0
 	LayoutWindow *lw = data;
-#endif
-	printf("FIXME: tools hide/float not implemented");
+
+	layout_tools_hide_toggle(lw);
 }
 
 static GtkWidget *layout_image_pop_menu(LayoutWindow *lw)
@@ -579,66 +586,72 @@ static GtkWidget *layout_image_pop_menu(LayoutWindow *lw)
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add(menu, _("Zoom in"), li_pop_menu_zoom_in_cb, lw);
-	menu_item_add(menu, _("Zoom out"), li_pop_menu_zoom_out_cb, lw);
-	menu_item_add(menu, _("Zoom 1:1"), li_pop_menu_zoom_1_1_cb, lw);
-	menu_item_add(menu, _("Fit image to window"), li_pop_menu_zoom_fit_cb, lw);
+	menu_item_add(menu, _("Zoom in"), GTK_SIGNAL_FUNC(li_pop_menu_zoom_in_cb), lw);
+	menu_item_add(menu, _("Zoom out"), GTK_SIGNAL_FUNC(li_pop_menu_zoom_out_cb), lw);
+	menu_item_add(menu, _("Zoom 1:1"), GTK_SIGNAL_FUNC(li_pop_menu_zoom_1_1_cb), lw);
+	menu_item_add(menu, _("Fit image to window"), GTK_SIGNAL_FUNC(li_pop_menu_zoom_fit_cb), lw);
 	menu_item_add_divider(menu);
 
-	submenu = submenu_add_edit(menu, &item, li_pop_menu_edit_cb, lw);
+	submenu = submenu_add_edit(menu, &item, GTK_SIGNAL_FUNC(li_pop_menu_edit_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 	menu_item_add_divider(submenu);
-	menu_item_add(submenu, _("Set as wallpaper"), li_pop_menu_wallpaper_cb, lw);
+	menu_item_add(submenu, _("Set as wallpaper"), GTK_SIGNAL_FUNC(li_pop_menu_wallpaper_cb), lw);
 
-	item = submenu_add_alter(menu, li_pop_menu_alter_cb, lw);
+	item = submenu_add_alter(menu, GTK_SIGNAL_FUNC(li_pop_menu_alter_cb), lw);
 
-	item = menu_item_add(menu, _("View in new window"), li_pop_menu_new_cb, lw);
+	item = menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(li_pop_menu_info_cb), lw);
+	if (!path) gtk_widget_set_sensitive(item, FALSE);
+
+	item = menu_item_add(menu, _("View in new window"), GTK_SIGNAL_FUNC(li_pop_menu_new_cb), lw);
 	if (!path || fullscreen) gtk_widget_set_sensitive(item, FALSE);
 
 	menu_item_add_divider(menu);
 
-	item = menu_item_add(menu, _("Copy..."), li_pop_menu_copy_cb, lw);
+	item = menu_item_add(menu, _("Copy..."), GTK_SIGNAL_FUNC(li_pop_menu_copy_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
-	item = menu_item_add(menu, _("Move..."), li_pop_menu_move_cb, lw);
+	item = menu_item_add(menu, _("Move..."), GTK_SIGNAL_FUNC(li_pop_menu_move_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
-	item = menu_item_add(menu, _("Rename..."), li_pop_menu_rename_cb, lw);
+	item = menu_item_add(menu, _("Rename..."), GTK_SIGNAL_FUNC(li_pop_menu_rename_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
-	item = menu_item_add(menu, _("Delete..."), li_pop_menu_delete_cb, lw);
+	item = menu_item_add(menu, _("Delete..."), GTK_SIGNAL_FUNC(li_pop_menu_delete_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 
 	menu_item_add_divider(menu);
 
 	if (layout_image_slideshow_active(lw))
 		{
-		menu_item_add(menu, _("Stop slideshow"), li_pop_menu_slide_stop_cb, lw);
+		menu_item_add(menu, _("Stop slideshow"), GTK_SIGNAL_FUNC(li_pop_menu_slide_stop_cb), lw);
 		if (layout_image_slideshow_paused(lw))
 			{
-			item = menu_item_add(menu, _("Continue slideshow"), li_pop_menu_slide_pause_cb, lw);
+			item = menu_item_add(menu, _("Continue slideshow"),
+					     GTK_SIGNAL_FUNC(li_pop_menu_slide_pause_cb), lw);
 			}
 		else
 			{
-			item = menu_item_add(menu, _("Pause slideshow"), li_pop_menu_slide_pause_cb, lw);
+			item = menu_item_add(menu, _("Pause slideshow"),
+					     GTK_SIGNAL_FUNC(li_pop_menu_slide_pause_cb), lw);
 			}
 		}
 	else
                 {
-		menu_item_add(menu, _("Start slideshow"), li_pop_menu_slide_start_cb, lw);
-		item = menu_item_add(menu, _("Pause slideshow"), li_pop_menu_slide_pause_cb, lw);
+		menu_item_add(menu, _("Start slideshow"), GTK_SIGNAL_FUNC(li_pop_menu_slide_start_cb), lw);
+		item = menu_item_add(menu, _("Pause slideshow"), GTK_SIGNAL_FUNC(li_pop_menu_slide_pause_cb), lw);
 		gtk_widget_set_sensitive(item, FALSE);
 		}
 
 	if (!fullscreen)
 		{
-		menu_item_add(menu, _("Full screen"), li_pop_menu_full_screen_cb, lw);
+		menu_item_add(menu, _("Full screen"), GTK_SIGNAL_FUNC(li_pop_menu_full_screen_cb), lw);
 		}
 	else
 		{
-		menu_item_add(menu, _("Exit full screen"), li_pop_menu_full_screen_cb, lw);
+		menu_item_add(menu, _("Exit full screen"), GTK_SIGNAL_FUNC(li_pop_menu_full_screen_cb), lw);
 		}
 
 	menu_item_add_divider(menu);
 
-	item = menu_item_add(menu, _("(Un)Hide file list"), li_pop_menu_hide_cb, lw);
+	item = menu_item_add_check(menu, _("Hide file list"), lw->tools_hidden,
+				   GTK_SIGNAL_FUNC(li_pop_menu_hide_cb), lw);
 	if (fullscreen) gtk_widget_set_sensitive(item, FALSE);
 
 	return menu;
@@ -786,16 +799,16 @@ static void layout_image_dnd_init(LayoutWindow *lw)
 			    dnd_file_drag_types, dnd_file_drag_types_count,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	gtk_signal_connect(GTK_OBJECT(lw->image->image), "drag_data_get",
-			   layout_image_dnd_get, lw);
+			   GTK_SIGNAL_FUNC(layout_image_dnd_get), lw);
 	gtk_signal_connect(GTK_OBJECT(lw->image->image), "drag_end",
-			   layout_image_dnd_end, lw);
+			   GTK_SIGNAL_FUNC(layout_image_dnd_end), lw);
 
 	gtk_drag_dest_set(lw->image->image,
 			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
 			  dnd_file_drop_types, dnd_file_drop_types_count,
                           GDK_ACTION_COPY);
 	gtk_signal_connect(GTK_OBJECT(lw->image->image), "drag_data_received",
-			   layout_image_dnd_receive, lw);
+			   GTK_SIGNAL_FUNC(layout_image_dnd_receive), lw);
 }
 
 
@@ -1206,6 +1219,7 @@ GtkWidget *layout_image_new(LayoutWindow *lw, const gchar *path)
 	if (!lw->image) 
 		{
 		lw->image = image_new(TRUE);
+		if (black_window_background) image_background_set_black(lw->image, TRUE);
 		image_set_update_func(lw->image, layout_image_update_cb, lw);
 		layout_image_set_buttons(lw);
 		layout_image_dnd_init(lw);
