@@ -17,6 +17,7 @@
 #include "collect.h"
 #include "collect-dlg.h"
 #include "dupe.h"
+#include "info.h"
 #include "layout_image.h"
 #include "preferences.h"
 #include "utilops.h"
@@ -302,6 +303,18 @@ static void layout_menu_alter_flip_cb(gpointer data, guint action, GtkWidget *wi
 	layout_image_alter(lw, ALTER_FLIP);
 }
 
+static void layout_menu_info_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+	GList *list;
+	const gchar *path = NULL;
+
+	list = layout_selection_list(lw);
+	if (!list) path = layout_image_get_path(lw);
+
+	info_window_new(path, list);
+}
+
 static void layout_menu_select_all_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
@@ -375,6 +388,20 @@ static void layout_menu_thumb_cb(gpointer data, guint action, GtkWidget *widget)
 	layout_thumb_set(lw, GTK_CHECK_MENU_ITEM(widget)->active);
 }
 
+static void layout_menu_tree_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	layout_views_set(lw, GTK_CHECK_MENU_ITEM(widget)->active, lw->icon_view);
+}
+
+static void layout_menu_icon_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	layout_views_set(lw, lw->tree_view, GTK_CHECK_MENU_ITEM(widget)->active);
+}
+
 static void layout_menu_refresh_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
@@ -384,20 +411,29 @@ static void layout_menu_refresh_cb(gpointer data, guint action, GtkWidget *widge
 
 static void layout_menu_float_cb(gpointer data, guint action, GtkWidget *widget)
 {
-#if 0
 	LayoutWindow *lw = data;
-#endif
 
-	printf("FIXME: implement float\n");
+	if (lw->tools_float != GTK_CHECK_MENU_ITEM(widget)->active)
+		{
+		layout_tools_float_toggle(lw);
+		}
 }
 
 static void layout_menu_hide_cb(gpointer data, guint action, GtkWidget *widget)
 {
-#if 0
 	LayoutWindow *lw = data;
-#endif
 
-	printf("FIXME: implement hide\n");
+	layout_tools_hide_toggle(lw);
+}
+
+static void layout_menu_toolbar_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	if (lw->toolbar_hidden != GTK_CHECK_MENU_ITEM(widget)->active)
+		{
+		layout_toolbar_toggle(lw);
+		}
 }
 
 static void layout_menu_slideshow_cb(gpointer data, guint action, GtkWidget *widget)
@@ -526,7 +562,7 @@ static void layout_menu_recent_update(LayoutWindow *lw)
 	while (list)
 		{
 		item = menu_item_add(menu, filename_from_path((gchar *)list->data),
-				     layout_menu_recent_cb, lw);
+				     GTK_SIGNAL_FUNC(layout_menu_recent_cb), lw);
 		gtk_object_set_data(GTK_OBJECT(item), "recent_index", GINT_TO_POINTER(n));
 		list = list->next;
 		n++;
@@ -602,6 +638,7 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 	{ N_("/Edit/editor8"),			"<control>8",	layout_menu_edit_cb,		MENU_EDIT_ACTION_OFFSET + 7, NULL },
 	{ N_("/Edit/sep1"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/Edit/_Adjust"),			NULL,		NULL,				0, "<Branch>" },
+	{ N_("/Edit/_Properties"),		"<control>P",	layout_menu_info_cb,		0, NULL },
 	{ N_("/Edit/Adjust/tear1"),		NULL,		NULL,				0, "<Tearoff>" },
 	{ N_("/Edit/Adjust/_Rotate clockwise"),		"bracketright",	layout_menu_alter_90_cb,	0, NULL },
 	{ N_("/Edit/Adjust/Rotate _counterclockwise"),	"bracketleft",	layout_menu_alter_90cc_cb,	0, NULL },
@@ -613,7 +650,7 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 	{ N_("/Edit/Select _none"),	"<control><shift>A",	layout_menu_unselect_all_cb,	0, NULL },
 	{ N_("/Edit/sep3"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/Edit/_Options..."),		"<control>O",	layout_menu_config_cb,		0, NULL },
-	{ N_("/Edit/_Remove old thumbnails"),	"<control>T",	layout_menu_remove_thumb_cb,	0, NULL },
+	{ N_("/Edit/_Remove old thumbnails"),	NULL,		layout_menu_remove_thumb_cb,	0, NULL },
 	{ N_("/Edit/sep4"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/Edit/Set as _wallpaper"),	"<control>W",	layout_menu_wallpaper_cb,	0, NULL },
 
@@ -629,11 +666,16 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 	{ N_("/View/_Thumbnails"),		"T",		layout_menu_thumb_cb,		0, "<CheckItem>" },
 
 	{ N_("/View/sep2"),			NULL,		NULL,				0, "<Separator>" },
-	{ N_("/View/_Refresh Lists"),		"R",		layout_menu_refresh_cb,		0, NULL },
-	{ N_("/View/(Un)_Float file list"),	"F",		layout_menu_float_cb,		0, NULL },
-	{ N_("/View/(Un)_Hide file list"),	"H",		layout_menu_hide_cb,		0, NULL },
+	{ N_("/View/Tr_ee"),			"<control>T",	layout_menu_tree_cb,		0, "<CheckItem>" },
+	{ N_("/View/I_cons"),			"<control>I",	layout_menu_icon_cb,		0, "<CheckItem>" },
+
 	{ N_("/View/sep3"),			NULL,		NULL,				0, "<Separator>" },
+	{ N_("/View/_Float file list"),		"F",		layout_menu_float_cb,		0, "<CheckItem>" },
+	{ N_("/View/_Hide file list"),		"H",		layout_menu_hide_cb,		0, NULL },
+	{ N_("/View/Hide tool_bar"),		NULL,		layout_menu_toolbar_cb,		0, "<CheckItem>" },
+	{ N_("/View/sep4"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/View/Toggle _slideshow"),	"S",		layout_menu_slideshow_cb,	0, NULL },
+	{ N_("/View/_Refresh Lists"),		"R",		layout_menu_refresh_cb,		0, NULL },
 
 	{ N_("/_Help"),				NULL,		NULL,				0, "<Branch>" },
 	{ N_("/Help/tear1"),			NULL,		NULL,				0, "<Tearoff>" },
@@ -723,11 +765,9 @@ static void layout_button_config_cb(GtkWidget *widget, gpointer data)
 
 static void layout_button_float_cb(GtkWidget *widget, gpointer data)
 {
-#if 0
 	LayoutWindow *lw = data;
-#endif
 
-	printf("FIXME: implement float\n");
+	layout_tools_float_toggle(lw);
 }
 
 GtkWidget *layout_button(GtkWidget *box, gchar **pixmap_data, gint toggle,
@@ -803,7 +843,24 @@ GtkWidget *layout_button_bar(LayoutWindow *lw)
  *-----------------------------------------------------------------------------
  * misc
  *-----------------------------------------------------------------------------
- */ 
+ */
+
+static void layout_util_sync_views(LayoutWindow *lw)
+{
+	GtkWidget *item;
+
+	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Tree");
+	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->tree_view);
+
+	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Icons");
+	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->icon_view);
+
+	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Float file list");
+	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->tools_float);
+
+	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Hide toolbar");
+	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->toolbar_hidden);
+}
 
 void layout_util_sync_thumb(LayoutWindow *lw)
 {
@@ -811,12 +868,16 @@ void layout_util_sync_thumb(LayoutWindow *lw)
 
 	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Thumbnails");
 	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->thumbs_enabled);
+	gtk_widget_set_sensitive(item, !lw->icon_view);
 
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(lw->thumb_button), lw->thumbs_enabled);
+
+	gtk_widget_set_sensitive(lw->thumb_button, !lw->icon_view);
 }
 
 void layout_util_sync(LayoutWindow *lw)
 {
+	layout_util_sync_views(lw);
 	layout_util_sync_thumb(lw);
 	layout_menu_recent_update(lw);
 	layout_menu_edit_update(lw);
