@@ -129,7 +129,8 @@ void help_window_show(const gchar *key)
 
 	help_window = help_window_new(_("Help - GQview"), "GQview", "help",
                                       GQVIEW_HELPDIR "/README", key);
-	gtk_signal_connect(GTK_OBJECT(help_window), "destroy", help_window_destroy_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(help_window), "destroy",
+			   GTK_SIGNAL_FUNC(help_window_destroy_cb), NULL);
 }
 
 
@@ -141,19 +142,39 @@ void help_window_show(const gchar *key)
 
 void start_editor_from_file(gint n, const gchar *path)
 {
+	gchar current_path[512];
+	gchar *success;
 	gchar *cmd;
 
 	if (!path) return;
 
 	cmd = g_strdup_printf("%s \"%s\" &", editor_command[n], path);
 	if (debug) printf(_("GQview running: %s\n"),cmd);
-	system(cmd);
 
+	success = getcwd(current_path, sizeof(current_path));
+
+	/* set working dir for the called application */
+	if (success)
+		{
+		gchar *base;
+		base = remove_level_from_path(path);
+		if (chdir(base) != 0) success = NULL;
+		g_free(base);
+		}
+
+	system(cmd);
 	g_free(cmd);
+
+	if (success)
+		{
+		chdir(success);
+		}
 }
 
 void start_editor_from_path_list(gint n, GList *list)
 {
+	gchar current_path[512];
+	gchar *success;
 	gchar *cmd;
 	gchar *buf;
 	GList *work;
@@ -165,7 +186,7 @@ void start_editor_from_path_list(gint n, GList *list)
 	while (work)
 		{
 		buf = cmd;
-		cmd = g_strconcat(buf, "\"", work->data, "\" ", NULL);
+		cmd = g_strconcat(buf, "\"", (gchar *)work->data, "\" ", NULL);
 		g_free(buf);
 		work = work->next;
 		}
@@ -173,9 +194,25 @@ void start_editor_from_path_list(gint n, GList *list)
 	cmd = g_strconcat(buf, "&", NULL);
 	g_free(buf);
 	if (debug) printf(_("GQview running: %s\n"), cmd);
-	system(cmd);
 
+        success = getcwd(current_path, sizeof(current_path));
+
+	/* set working dir for the called application */
+	if (success)
+		{
+		gchar *base;
+		base = remove_level_from_path((gchar *)list->data);
+		if (chdir(base) != 0) success = NULL;
+		g_free(base);
+		}
+
+	system(cmd);
 	g_free(cmd);
+
+	if (success)
+		{
+		chdir(success);
+		}
 }
 
 /*
@@ -436,18 +473,10 @@ static void exit_gqview_final(void)
 	thumbnails_enabled = layout_thumb_get(NULL);
 	layout_sort_get(NULL, &file_sort_method, &file_sort_ascending);
 
-#if 0
-	if (!toolwindow || (toolwindow && GTK_WIDGET_VISIBLE(toolwindow)) )
-		{
-		main_window_vdivider_pos = dir_clist->allocation.height;
-		}
-
-	if (toolwindow && GTK_WIDGET_VISIBLE(toolwindow))
-		{
-		gdk_window_get_root_origin (toolwindow->window, &float_window_x, &float_window_y);
-		gdk_window_get_size(toolwindow->window, &float_window_w, &float_window_h);
-		}
-#endif
+	layout_geometry_get_tools(NULL, &float_window_x, &float_window_y,
+				  &float_window_w, &float_window_h, &float_window_divider);
+	layout_tools_float_get(NULL, &tools_float, &tools_hidden);
+	toolbar_hidden = layout_toolbar_hidden(NULL);
 
 	save_options();
 	keys_save();
@@ -564,7 +593,7 @@ int main (int argc, char *argv[])
 
 	filter_rebuild();
 
-	lw = layout_new(NULL);
+	lw = layout_new(NULL, tools_float, tools_hidden);
 	layout_sort_set(lw, file_sort_method, file_sort_ascending);
 	layout_set_path(lw, path);
 
