@@ -42,7 +42,7 @@ static GtkWidget *add_menu_item(GtkWidget *menu, gchar *label, GtkAccelGroup *ac
 
 	item = gtk_menu_item_new_with_label(label);
 	gtk_widget_add_accelerator (item, "activate", accel_group, accel_key, accel_mods, GTK_ACCEL_VISIBLE);
-	gtk_signal_connect (GTK_OBJECT (item), "activate",(GtkSignalFunc) func, data);
+	gtk_signal_connect(GTK_OBJECT(item), "activate", func, data);
 	gtk_menu_append(GTK_MENU(menu), item);
 	gtk_widget_show(item);
 
