@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -14,28 +14,17 @@
 #define MENU_H
 
 
-void update_edit_menus(GtkAccelGroup *accel_grp);
-
 gpointer submenu_item_get_data(GtkWidget *menu);
 
 GtkWidget *submenu_add_edit(GtkWidget *menu, GtkWidget **menu_item, GtkSignalFunc func, gpointer data);
 
 gchar *sort_type_get_text(SortType method);
 GtkWidget *submenu_add_sort(GtkWidget *menu, GtkSignalFunc func, gpointer data,
-			    gint include_none, gint include_path);
+			    gint include_none, gint include_path,
+			    gint show_current, SortType type);
 
 gchar *alter_type_get_text(AlterType type);
 GtkWidget *submenu_add_alter(GtkWidget *menu, GtkSignalFunc func, gpointer data);
 
-void recent_menu_add_path(gchar *path);
-
-GtkWidget *create_menu_bar(GtkAccelGroup *accel_grp);
-
-GtkWidget *menu_popup_dir_list(gint active);
-GtkWidget *menu_popup_file_list(gint active);
-GtkWidget *menu_popup_image(void);
-
-GtkWidget *create_button_bar(GtkTooltips *tooltips);
-
 
 #endif
