@@ -15,6 +15,7 @@
 
 #include "cache_maint.h"
 #include "filelist.h"
+#include "img-view.h"
 #include "layout_config.h"
 #include "layout_util.h"
 #include "ui_tabcomp.h"
@@ -53,6 +54,7 @@ static gint enable_delete_key_c;
 static gint restore_tool_c;
 static gint save_window_positions_c;
 static gint zoom_mode_c;
+static gint two_pass_zoom_c;
 static gint fit_window_c;
 static gint limit_window_size_c;
 static gint zoom_to_fit_expands_c;
@@ -97,6 +99,10 @@ static gint enable_read_ahead_c;
 
 static gint place_dialogs_under_mouse_c;
 
+static gint black_window_background_c;
+
+static gint dupe_custom_threshold_c;
+
 static GtkWidget *configwindow = NULL;
 static GtkWidget *startup_path_entry;
 static GtkWidget *custom_filter_entry;
@@ -124,6 +130,11 @@ static void slideshow_delay_cb(GtkObject *adj, gpointer data)
 	slideshow_delay_c = (gint)GTK_ADJUSTMENT(adj)->value;
 }
 
+static void dupe_custom_cb(GtkObject *adj, gpointer data)
+{
+	dupe_custom_threshold_c = (gint)GTK_ADJUSTMENT(adj)->value;
+}
+
 static void zoom_mode_original_cb(GtkWidget *widget, gpointer data)
 {
 	if (GTK_TOGGLE_BUTTON (widget)->active)
@@ -230,6 +241,7 @@ static void config_window_apply(void)
 	restore_tool = restore_tool_c;
 	save_window_positions = save_window_positions_c;
 	zoom_mode = zoom_mode_c;
+	two_pass_zoom = two_pass_zoom_c;
 	fit_window = fit_window_c;
 	limit_window_size = limit_window_size_c;
 	zoom_to_fit_expands = zoom_to_fit_expands_c;
@@ -275,6 +287,16 @@ static void config_window_apply(void)
 
 	place_dialogs_under_mouse = place_dialogs_under_mouse_c;
 
+	if (black_window_background != black_window_background_c)
+		{
+		black_window_background = black_window_background_c;
+
+		layout_colors_update();
+		view_window_colors_update();
+		}
+
+	dupe_custom_threshold = dupe_custom_threshold_c;
+
 	tree_descend_subdirs = tree_descend_subdirs_c;
 
 	buf = layout_config_get(layout_widget, &new_style);
@@ -314,9 +336,10 @@ static void config_window_close_cb(GtkWidget *widget, gpointer data)
 	configwindow = NULL;
 }
 
-static void config_window_destroy(GtkWidget *w, GdkEvent *event, gpointer data)
+static gint config_window_delete(GtkWidget *w, GdkEventAny *event, gpointer data)
 {
 	config_window_close_cb(NULL, NULL);
+	return TRUE;
 }
 
 static void config_window_ok_cb(GtkWidget *widget, gpointer data)
@@ -349,7 +372,8 @@ static void add_check_button(gint option, gint *option_c, gchar *text, GtkWidget
 	button = gtk_check_button_new_with_label (text);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), option);
-	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) check_button_cb, option_c);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(check_button_cb), option_c);
 	gtk_widget_show(button);
 }
 
@@ -366,7 +390,8 @@ static void add_menu_item(GtkWidget *menu, gchar *text, gint *option_c, gint typ
 
 	item = gtk_menu_item_new_with_label(text);
 	gtk_object_set_data(GTK_OBJECT(item), "value", GINT_TO_POINTER(type));
-	gtk_signal_connect(GTK_OBJECT(item), "activate", (GtkSignalFunc) quality_menu_cb, option_c);
+	gtk_signal_connect(GTK_OBJECT(item), "activate",
+			   GTK_SIGNAL_FUNC(quality_menu_cb), option_c);
 	gtk_menu_append(GTK_MENU (menu), item);
 	gtk_widget_show(item);
 }
@@ -491,7 +516,8 @@ static void add_thumb_size_menu(gchar *text, GtkWidget *box)
 			item = gtk_menu_item_new_with_label(buf);
 			g_free(buf);
 
-			gtk_signal_connect(GTK_OBJECT(item), "activate", (GtkSignalFunc) thumb_size_menu_cb, GINT_TO_POINTER(i));
+			gtk_signal_connect(GTK_OBJECT(item), "activate",
+					   GTK_SIGNAL_FUNC(thumb_size_menu_cb), GINT_TO_POINTER(i));
 			gtk_menu_append(GTK_MENU (menu), item);
 			gtk_widget_show(item);
 			if (thumb_max_width >= w && thumb_max_width >= h) row = i;
@@ -529,11 +555,12 @@ static void config_window_create(void)
 	gint i;
 
 	configwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_signal_connect (GTK_OBJECT (configwindow), "delete_event",(GtkSignalFunc) config_window_destroy, NULL);
-	gtk_window_set_policy (GTK_WINDOW (configwindow), FALSE, FALSE, FALSE);
-	gtk_window_set_title (GTK_WINDOW (configwindow), _("GQview configuration"));
-	gtk_window_set_wmclass(GTK_WINDOW (configwindow), "config", "GQview");
-	gtk_container_border_width (GTK_CONTAINER (configwindow), 5);
+	gtk_signal_connect(GTK_OBJECT (configwindow), "delete_event",
+			   GTK_SIGNAL_FUNC(config_window_delete), NULL);
+	gtk_window_set_policy(GTK_WINDOW(configwindow), FALSE, FALSE, FALSE);
+	gtk_window_set_title(GTK_WINDOW(configwindow), _("GQview configuration"));
+	gtk_window_set_wmclass(GTK_WINDOW(configwindow), "config", "GQview");
+	gtk_container_border_width(GTK_CONTAINER (configwindow), 5);
 
 	window_set_icon(configwindow, (const gchar **)config_xpm, NULL);
 
@@ -548,20 +575,23 @@ static void config_window_create(void)
 	gtk_widget_show(hbox);
 
 	button = gtk_button_new_with_label(_("Ok"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) config_window_ok_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(config_window_ok_cb), NULL);
 	gtk_container_add(GTK_CONTAINER(hbox), button);
 	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 	gtk_widget_grab_default(button);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Apply"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) config_window_apply_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(config_window_apply_cb), NULL);
 	gtk_container_add(GTK_CONTAINER(hbox), button);
 	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Cancel"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) config_window_close_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(config_window_close_cb), NULL);
 	gtk_container_add(GTK_CONTAINER(hbox), button);
 	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 	gtk_widget_show(button);
@@ -606,7 +636,8 @@ static void config_window_create(void)
 	gtk_widget_show(tabcomp);
 
 	button = gtk_button_new_with_label (_("Use current"));
-	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) startup_path_set_current, NULL);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(startup_path_set_current), NULL);
 	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
@@ -629,7 +660,8 @@ static void config_window_create(void)
 	add_check_button(enable_thumb_caching, &enable_thumb_caching_c,
 			 _("Cache thumbnails"), hbox);
 	button = gtk_button_new_with_label(_("Clear cache"));
-	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) cache_clear_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(cache_clear_cb), NULL);
 	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
@@ -658,11 +690,12 @@ static void config_window_create(void)
 
 	slideshow_delay_c = slideshow_delay;
 	adj = gtk_adjustment_new((float)slideshow_delay_c, 1.0, 1200.0, 1, 1, 1);
-        spin = gtk_spin_button_new( GTK_ADJUSTMENT(adj), 1, 0 );
-        gtk_box_pack_start( GTK_BOX(hbox), spin, FALSE, FALSE, 5);
-        gtk_signal_connect( GTK_OBJECT(adj),"value_changed",GTK_SIGNAL_FUNC(slideshow_delay_cb), NULL);
-        gtk_spin_button_set_update_policy( GTK_SPIN_BUTTON(spin),GTK_UPDATE_ALWAYS );
-        gtk_widget_show(spin);
+	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+	gtk_box_pack_start( GTK_BOX(hbox), spin, FALSE, FALSE, 5);
+	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+			   GTK_SIGNAL_FUNC(slideshow_delay_cb), NULL);
+	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
+	gtk_widget_show(spin);
 	
 	hbox = gtk_hbox_new(FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
@@ -695,6 +728,23 @@ static void config_window_create(void)
 	add_check_button(enable_delete_key, &enable_delete_key_c,
 			 _("Enable Delete key"), vbox1);
 
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	label = gtk_label_new(_("Custom similarity threshold:"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	gtk_widget_show(label);
+
+	dupe_custom_threshold_c = dupe_custom_threshold;
+	adj = gtk_adjustment_new((float)dupe_custom_threshold_c, 0.0, 100.0, 1, 1, 1);
+	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 5);
+	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+			   GTK_SIGNAL_FUNC(dupe_custom_cb), NULL);
+	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
+	gtk_widget_show(spin);
+
 	/* image tab */
 
 	frame = gtk_frame_new(NULL);
@@ -718,11 +768,8 @@ static void config_window_create(void)
 	gtk_widget_show(vbox1);
 
 	add_dither_menu(dither_quality, &dither_quality_c, _("Dithering method:"), vbox1);
-#ifdef GDK_PIXBUF_TILES_BROKEN
-	add_quality_menu(zoom_quality, &zoom_quality_c, _("Zoom (scaling):"), vbox1, FALSE);
-#else
-	add_quality_menu(zoom_quality, &zoom_quality_c, _("Zoom (scaling):"), vbox1, TRUE);
-#endif
+	add_quality_menu(zoom_quality, &zoom_quality_c, _("Zoom (scaling):"), vbox1, !(GDK_PIXBUF_TILES_BROKEN));
+	add_check_button(two_pass_zoom, &two_pass_zoom_c, _("Two pass zooming"), vbox1);
 
 	frame = gtk_frame_new(_("When new image is selected:"));
 	gtk_container_border_width (GTK_CONTAINER (frame), 5);
@@ -736,19 +783,22 @@ static void config_window_create(void)
 	zoom_mode_c = zoom_mode;
 	radiobuttongroup = gtk_radio_button_new_with_label (NULL, _("Zoom to original size"));
 	if (zoom_mode == ZOOM_RESET_ORIGINAL) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(radiobuttongroup), 1);
-	gtk_signal_connect (GTK_OBJECT(radiobuttongroup),"clicked",(GtkSignalFunc) zoom_mode_original_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(radiobuttongroup), "clicked",
+			   GTK_SIGNAL_FUNC(zoom_mode_original_cb), NULL);
 	gtk_box_pack_start(GTK_BOX(vbox1), radiobuttongroup, FALSE, FALSE, 0);
 	gtk_widget_show(radiobuttongroup);
 
 	button = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiobuttongroup)),_("Fit image to window"));
 	if (zoom_mode == ZOOM_RESET_FIT_WINDOW) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), 1);
-	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) zoom_mode_fit_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(zoom_mode_fit_cb), NULL);
 	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiobuttongroup)),_("Leave Zoom at previous setting"));
 	if (zoom_mode == ZOOM_RESET_NONE) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), 1);
-	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) zoom_mode_none_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(zoom_mode_none_cb), NULL);
 	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
@@ -767,7 +817,8 @@ static void config_window_create(void)
 	adj = gtk_adjustment_new((float)tile_cache_max_c, 0.0, 128.0, 1.0, 1.0, 1.0);
 	spin = gtk_spin_button_new( GTK_ADJUSTMENT(adj), 1, 0 );
 	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 5);
-	gtk_signal_connect(GTK_OBJECT(adj), "value_changed", GTK_SIGNAL_FUNC(tile_cache_max_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+			   GTK_SIGNAL_FUNC(tile_cache_max_cb), NULL);
 	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
 	gtk_widget_show(spin);
 
@@ -783,7 +834,8 @@ static void config_window_create(void)
 	adj = gtk_adjustment_new(zoom_increment_c ? (float)zoom_increment_c / 10.0 : 1.0, 0.1, 4.0, 0.1, 0.1, 0.1);
 	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 1);
 	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 5);
-	gtk_signal_connect(GTK_OBJECT(adj), "value_changed", GTK_SIGNAL_FUNC(zoom_increment_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+			   GTK_SIGNAL_FUNC(zoom_increment_cb), NULL);
 	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
 	gtk_widget_show(spin);
 
@@ -792,6 +844,9 @@ static void config_window_create(void)
 	add_check_button(mousewheel_scrolls, &mousewheel_scrolls_c,
 			 _("Mouse wheel scrolls image"), vbox);
 
+	add_check_button(black_window_background, &black_window_background_c,
+			 _("Black background"), vbox);
+
 	/* window tab */
 
 	frame = gtk_frame_new(NULL);
@@ -824,7 +879,8 @@ static void config_window_create(void)
 	adj = gtk_adjustment_new((float)max_window_size_c, 10.0, 150.0, 1.0, 1.0, 1.0);
         spin = gtk_spin_button_new( GTK_ADJUSTMENT(adj), 1, 0);
         gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 5);
-        gtk_signal_connect(GTK_OBJECT(adj), "value_changed", GTK_SIGNAL_FUNC(max_window_size_cb), NULL);
+        gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+			   GTK_SIGNAL_FUNC(max_window_size_cb), NULL);
         gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
         gtk_widget_show(spin);
 
@@ -1054,7 +1110,8 @@ void show_about_window(void)
 
 	about = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_wmclass(GTK_WINDOW(about), "about", "GQview");
-	gtk_signal_connect(GTK_OBJECT(about), "delete_event", GTK_SIGNAL_FUNC(about_delete_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(about), "delete_event",
+			   GTK_SIGNAL_FUNC(about_delete_cb), NULL);
 
 	gtk_window_set_title(GTK_WINDOW(about), "About - GQview");
 	gtk_container_border_width(GTK_CONTAINER(about), 5);
