@@ -163,6 +163,8 @@ void save_options(void)
 	if (zoom_mode == ZOOM_RESET_FIT_WINDOW) fprintf(f,"fit\n");
 	if (zoom_mode == ZOOM_RESET_NONE) fprintf(f,"dont_change\n");
 
+	write_bool_option(f, "two_pass_scaling", two_pass_zoom);
+
 	write_bool_option(f, "zoom_to_fit_allow_expand", zoom_to_fit_expands);
 	fprintf(f,"\n");
 
@@ -193,6 +195,8 @@ void save_options(void)
 	write_bool_option(f, "tools_hidden", tools_hidden);
 	write_bool_option(f, "restore_tool_state", restore_tool);
 
+	write_bool_option(f, "toolbar_hidden", toolbar_hidden);
+
 	write_bool_option(f, "mouse_wheel_scrolls", mousewheel_scrolls);
 	write_bool_option(f, "in_place_rename", enable_in_place_rename);
 
@@ -206,9 +210,13 @@ void save_options(void)
 
 	write_int_option(f, "zoom_increment", zoom_increment);
 
-	write_int_option(f, "enable_read_ahead", enable_read_ahead);
+	write_bool_option(f, "enable_read_ahead", enable_read_ahead);
 
-	write_int_option(f, "place_dialogs_under_mouse", place_dialogs_under_mouse);
+	write_bool_option(f, "place_dialogs_under_mouse", place_dialogs_under_mouse);
+
+	write_bool_option(f, "black_window_background", black_window_background);
+
+	write_int_option(f, "custom_similarity_threshold", dupe_custom_threshold);
 
 	fprintf(f,"\n##### Slideshow Options #####\n\n");
 
@@ -264,6 +272,7 @@ void save_options(void)
 	write_int_option(f, "float_window_y", float_window_y);
 	write_int_option(f, "float_window_width", float_window_w);
 	write_int_option(f, "float_window_height", float_window_h);
+	write_int_option(f, "float_window_divider", float_window_divider);
 	write_int_option(f, "divider_position_h", window_hdivider_pos);
 	write_int_option(f, "divider_position_v", window_vdivider_pos);
 
@@ -316,11 +325,11 @@ void load_options(void)
 		while (s_buf[c] == 8 && c < l) c++;
 		while (s_buf[c] == ' ' && c < l) c++;
 		s_buf_ptr = s_buf + c;
-		strcpy(value_all,s_buf_ptr);
+		strncpy(value_all, s_buf_ptr, sizeof(value_all));
 		while (s_buf[c] != 8 && s_buf[c] != ' ' && s_buf[c] != '\n' && c < l) c++;
 		s_buf[c] = '\0';
-		strcpy(option,s_buf);
-		strcpy(value,s_buf_ptr);
+		strncpy(option, s_buf, sizeof(option));
+		strncpy(value, s_buf_ptr, sizeof(value));
 
 		/* general options */
 
@@ -347,6 +356,8 @@ void load_options(void)
                         if (!strcasecmp(value,"fit")) zoom_mode = ZOOM_RESET_FIT_WINDOW;
                         if (!strcasecmp(value,"dont_change")) zoom_mode = ZOOM_RESET_NONE;
                         }
+		two_pass_zoom = read_bool_option(f, option,
+			"two_pass_scaling", value, two_pass_zoom);
 		zoom_to_fit_expands = read_bool_option(f, option,
 			"zoom_to_fit_allow_expand", value, zoom_to_fit_expands);
 
@@ -389,6 +400,9 @@ void load_options(void)
 		restore_tool = read_bool_option(f, option,
 			"restore_tool_state", value, restore_tool);
 
+		toolbar_hidden = read_bool_option(f, option,
+			"toolbar_hidden", value, toolbar_hidden);
+
 		mousewheel_scrolls = read_bool_option(f, option,
 			"mouse_wheel_scrolls", value, mousewheel_scrolls);
 		enable_in_place_rename = read_bool_option(f, option,
@@ -410,12 +424,18 @@ void load_options(void)
 		zoom_increment = read_int_option(f, option,
 			"zoom_increment", value, zoom_increment);
 
-		enable_read_ahead = read_int_option(f, option,
+		enable_read_ahead = read_bool_option(f, option,
 			"enable_read_ahead", value, enable_read_ahead);
 
-		place_dialogs_under_mouse = read_int_option(f, option,
+		place_dialogs_under_mouse = read_bool_option(f, option,
 			"place_dialogs_under_mouse", value, place_dialogs_under_mouse);
 
+		black_window_background = read_bool_option(f, option,
+			"black_window_background", value, black_window_background);
+
+		dupe_custom_threshold = read_int_option(f, option,
+			"custom_similarity_threshold", value, dupe_custom_threshold);
+
 		/* slideshow options */
 
 		slideshow_delay = read_int_option(f, option,
@@ -529,6 +549,8 @@ void load_options(void)
 			"float_window_width", value, float_window_w);
 		float_window_h = read_int_option(f, option,
 			"float_window_height", value, float_window_h);
+		float_window_divider = read_int_option(f, option,
+			"float_window_divider", value, float_window_divider);
 		window_hdivider_pos = read_int_option(f, option,
 			"divider_position_h", value, window_hdivider_pos);
 		window_vdivider_pos = read_int_option(f, option,
@@ -539,19 +561,21 @@ void load_options(void)
 	fclose(f);
 	g_free(rc_path);
 
-#ifdef GDK_PIXBUF_TILES_BROKEN
-	if ((GdkInterpType)zoom_quality == GDK_INTERP_TILES)
+	if (GDK_PIXBUF_TILES_BROKEN)
 		{
-		/* this is not so bad, the new and old gdk-pixbufs/GQviews are about
-		 * the same speed for these two scale qualities.
-		 * the message does not need translation, it planned for removal, whether
-		 * it will be because of a fixed gdk-pixbuf or simpy dropped, is TBD.
-		 */
-		zoom_quality = (gint)GDK_INTERP_BILINEAR;
-		printf("Important message:\n");
-		printf("Newer gdk-pixbuf has broken tile quality scaling, changing to bilinear.\n");
+		printf("Your version of gdk-pixbuf has buggy scaling, TILES zoom quality disabled.");
+		printf(" (upgrade gdk-pixbuf to at least 0.16.0 to fix this.\n");
+		if ((GdkInterpType)zoom_quality == GDK_INTERP_TILES)
+			{
+			/* this is not so bad, the new and old gdk-pixbufs/GQviews are about
+			 * the same speed for these two scale qualities.
+			 * the message does not need translation, it planned for removal, whether
+			 * it will be because of a fixed gdk-pixbuf or simpy dropped, is TBD.
+			 */
+			zoom_quality = (gint)GDK_INTERP_BILINEAR;
+			printf("Important message:\n");
+			printf("Newer gdk-pixbuf has broken tile quality scaling, changing to bilinear.\n");
+			}
 		}
-#endif
-
 }
 
