@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -16,36 +16,9 @@
 
 /*
  * It works like this, it uses path_list, if that does not exist, it uses
- * CollectionData, then finally falls back to the file listing.
+ * CollectionData, then finally falls back to the layout listing.
  */
 
-
-typedef struct _SlideShowData SlideShowData;
-struct _SlideShowData
-{
-	ImageWindow *imd;
-
-	GList *path_list;
-	CollectionData *cd;
-	gchar *filelist_path;
-
-	GList *list;
-	GList *list_done;
-
-	gchar *slide_path;
-
-	gint slide_count;
-	gint timeout_id;
-
-	gint from_selection;
-
-	void (*stop_func)(SlideShowData *, gpointer);
-	gpointer stop_data;
-
-	gint paused;
-};
-
-
 void slideshow_free(SlideShowData *ss);
 
 gint slideshow_should_continue(SlideShowData *ss);
@@ -58,10 +31,8 @@ SlideShowData *slideshow_start_from_path_list(ImageWindow *imd, GList *list,
 SlideShowData *slideshow_start_from_collection(ImageWindow *imd, CollectionData *cd,
 					       void (*stop_func)(SlideShowData *, gpointer), gpointer stop_data,
 					       CollectInfo *start_info);
-SlideShowData *slideshow_start(ImageWindow *imd,
-			       void (*stop_func)(SlideShowData *, gpointer), gpointer stop_data,
-			       gint start_point);
-
+SlideShowData *slideshow_start(ImageWindow *imd, LayoutWindow *lw, gint start_point,
+			       void (*stop_func)(SlideShowData *, gpointer), gpointer stop_data);
 
 gint slideshow_paused(SlideShowData *ss);
 void slideshow_pause_set(SlideShowData *ss, gint paused);
@@ -69,3 +40,4 @@ gint slideshow_pause_toggle(SlideShowData *ss);
 
 
 #endif
+
