@@ -242,9 +242,12 @@ struct _ImageWindow
 	gint tile_cache_size;	/* allocated size of pixmaps/pixbufs */
 	GList *draw_queue;	/* list of areas to redraw */
 
+	GList *draw_queue_2pass;/* list when 2 pass is enabled */
+
 	ImageLoader *il;
 
 	gint draw_idle_id;
+	gint draw_idle_high;	/* current idle_id has high priority */
 
 	gint in_drag;
 	gint drag_last_x;
@@ -328,8 +331,14 @@ struct _LayoutWindow
 	GtkWidget *tools;
 	GtkWidget *tools_pane;
 
+	gint tools_float;
+	gint tools_hidden;
+
 	/* toolbar */
 
+	GtkWidget *toolbar;
+	gint toolbar_hidden;
+
 	GtkWidget *thumb_button;
 	gint thumbs_enabled;
 
@@ -371,6 +380,12 @@ struct _LayoutWindow
 	/* full screen */
 
 	FullScreenData *full_screen;
+
+	/* dividers */
+
+	gint div_h;
+	gint div_v;
+	gint div_float;
 };
 
 struct _ViewDirList
@@ -386,6 +401,8 @@ struct _ViewDirList
 	gint drop_row;
 	GList *drop_list;
 
+	gint drop_scroll_id;
+
 	/* func list */
 	void (*select_func)(ViewDirList *vdl, const gchar *path, gpointer data);
 	gpointer select_data;
@@ -409,6 +426,7 @@ struct _ViewDirTree
 	GtkCTreeNode *drop_node;
 	GList *drop_list;
 
+	gint drop_scroll_id;
 	gint drop_expand_id;
 	gint drop_row;
 
