@@ -504,7 +504,7 @@ static void tab_completion_popup_list(TabCompData *td, GList *list)
 		gchar *name = work->data;
 		GtkWidget *item;
 
-		item = menu_item_add(menu, name, tab_completion_popup_cb, name);
+		item = menu_item_add(menu, name, GTK_SIGNAL_FUNC(tab_completion_popup_cb), name);
 		gtk_object_set_data(GTK_OBJECT(item), "tab_completion_data", td);
 
 		work = work->next;
@@ -548,7 +548,7 @@ static gint simple_sort(gconstpointer a, gconstpointer b)
 
 static gint tab_completion_do(TabCompData *td)
 {
-	gchar *entry_text = gtk_entry_get_text(GTK_ENTRY(td->entry));
+	const gchar *entry_text = gtk_entry_get_text(GTK_ENTRY(td->entry));
 	const gchar *entry_file;
 	gchar *entry_dir;
 	gchar *ptr;
@@ -785,7 +785,8 @@ static GtkWidget *tab_completion_create_complete_button(GtkWidget *entry)
 
 	button = gtk_button_new();
 	GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) tab_completion_button_pressed, entry);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(tab_completion_button_pressed), entry);
 	
 	icon = gtk_pixmap_new(pixmap, mask);
 	gtk_container_add(GTK_CONTAINER(button), icon);
@@ -932,9 +933,9 @@ void tab_completion_add_to_entry(GtkWidget *entry, void (*enter_func)(const gcha
 	gtk_object_set_data(GTK_OBJECT(td->entry), "tab_completion_data", td);
 
 	gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
-			(GtkSignalFunc) tab_completion_key_pressed, td);
+			   GTK_SIGNAL_FUNC(tab_completion_key_pressed), td);
 	gtk_signal_connect(GTK_OBJECT(entry), "destroy",
-			(GtkSignalFunc) tab_completion_destroy, td);
+			   GTK_SIGNAL_FUNC(tab_completion_destroy), td);
 }
 
 void tab_completion_add_tab_func(GtkWidget *entry, void (*tab_func)(const gchar *, gpointer), gpointer data)
