@@ -104,17 +104,19 @@ static void generic_dialog_show_cb(GtkWidget *widget, gpointer data)
 	GenericDialog *gd = data;
 	if (gd->cancel_cb) generic_dialog_add(gd, _("Cancel"), gd->cancel_cb, FALSE);
 
-	gtk_signal_disconnect_by_func(GTK_OBJECT(widget), generic_dialog_show_cb, gd);
+	gtk_signal_disconnect_by_func(GTK_OBJECT(widget),
+				      GTK_SIGNAL_FUNC(generic_dialog_show_cb), gd);
 }
 
-void generic_dialog_add(GenericDialog *gd, const gchar *text,
-			void (*func_cb)(GenericDialog *, gpointer), gint is_default)
+GtkWidget * generic_dialog_add(GenericDialog *gd, const gchar *text,
+			       void (*func_cb)(GenericDialog *, gpointer), gint is_default)
 {
 	GtkWidget *button;
 	button = gtk_button_new_with_label(text);
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 	gtk_object_set_data(GTK_OBJECT(button), "dialog_function", func_cb);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", generic_dialog_click_cb, gd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(generic_dialog_click_cb), gd);
 
 	gtk_container_add(GTK_CONTAINER(gd->hbox), button);
 
@@ -126,6 +128,8 @@ void generic_dialog_add(GenericDialog *gd, const gchar *text,
 		}
 
 	gtk_widget_show(button);
+
+	return button;
 }
 
 static void generic_dialog_setup(GenericDialog *gd,
@@ -176,7 +180,8 @@ static void generic_dialog_setup(GenericDialog *gd,
 	gtk_box_pack_start(GTK_BOX(vbox), gd->hbox, FALSE, FALSE, 0);
 	gtk_widget_show(gd->hbox);
 
-	gtk_signal_connect(GTK_OBJECT(gd->dialog), "show", GTK_SIGNAL_FUNC(generic_dialog_show_cb), gd);
+	gtk_signal_connect(GTK_OBJECT(gd->dialog), "show",
+			   GTK_SIGNAL_FUNC(generic_dialog_show_cb), gd);
 }
 
 GenericDialog *generic_dialog_new(const gchar *title, const gchar *message,
@@ -242,11 +247,11 @@ FileDialog *file_dialog_new(const gchar *title, const gchar *message,
 	return fd;
 }
 
-void file_dialog_add_button(FileDialog *fd, const gchar *text,
-			    void (*func_cb)(FileDialog *, gpointer), gint is_default)
+GtkWidget *file_dialog_add_button(FileDialog *fd, const gchar *text,
+				  void (*func_cb)(FileDialog *, gpointer), gint is_default)
 {
-	generic_dialog_add(GENERIC_DIALOG(fd), text,
-			   (void *)func_cb, is_default);
+	return generic_dialog_add(GENERIC_DIALOG(fd), text,
+				  (void *)func_cb, is_default);
 }
 
 static void file_dialog_entry_cb(GtkWidget *widget, gpointer data)
@@ -319,7 +324,8 @@ void file_dialog_add_path_widgets(FileDialog *fd, const gchar *default_path, con
 		gtk_entry_set_position(GTK_ENTRY(fd->entry), strlen(fd->dest_path));
 		}
 
-	gtk_signal_connect(GTK_OBJECT(fd->entry), "changed", GTK_SIGNAL_FUNC(file_dialog_entry_cb), fd);
+	gtk_signal_connect(GTK_OBJECT(fd->entry), "changed",
+			   GTK_SIGNAL_FUNC(file_dialog_entry_cb), fd);
 }
 
 void file_dialog_add_filter(FileDialog *fd, const gchar *filter, const gchar *filter_desc, gint set)
