@@ -83,6 +83,16 @@ struct _FileDataMult
 	gchar *source;
 	gchar *dest;
 	gint copy;
+
+	gint rename;
+	gint rename_auto;
+	gint rename_all;
+
+	GtkWidget *rename_box;
+	GtkWidget *rename_entry;
+	GtkWidget *rename_auto_box;
+
+	GtkWidget *yes_all_button;
 };
 
 typedef struct _FileDataSingle FileDataSingle;
@@ -92,6 +102,13 @@ struct _FileDataSingle
 	gchar *source;
 	gchar *dest;
 	gint copy;
+
+	gint rename;
+	gint rename_auto;
+
+	GtkWidget *rename_box;
+	GtkWidget *rename_entry;
+	GtkWidget *rename_auto_box;
 };
 
 /*
@@ -267,6 +284,41 @@ static void file_util_move_multiple_ok_cb(GenericDialog *gd, gpointer data)
 	FileDataMult *fdm = data;
 
 	fdm->confirmed = TRUE;
+
+	if (fdm->rename_auto)
+		{
+		gchar *buf;
+
+		buf = unique_filename_simple(fdm->dest);
+		if (buf)
+			{
+			g_free(fdm->dest);
+			fdm->dest = buf;
+			}
+		else
+			{
+			/* unique failed? well, return to the overwrite prompt :( */
+			fdm->confirmed = FALSE;
+			}
+		}
+	else if (fdm->rename)
+		{
+		const gchar *name;
+
+		name = gtk_entry_get_text(GTK_ENTRY(fdm->rename_entry));
+		if (strlen(name) == 0 ||
+		    strcmp(name, filename_from_path(fdm->source)) == 0)
+			{
+			fdm->confirmed = FALSE;
+			}
+		else
+			{
+			g_free(fdm->dest);
+			fdm->dest = concat_dir_and_file(fdm->dest_base, name);
+			fdm->confirmed = !isname(fdm->dest);
+			}
+		}
+
 	file_util_move_multiple(fdm);
 }
 
@@ -275,6 +327,9 @@ static void file_util_move_multiple_all_cb(GenericDialog *gd, gpointer data)
 	FileDataMult *fdm = data;
 
 	fdm->confirm_all = TRUE;
+
+	if (fdm->rename_auto) fdm->rename_all = TRUE;
+
 	file_util_move_multiple(fdm);
 }
 
@@ -293,6 +348,36 @@ static void file_util_move_multiple_cancel_cb(GenericDialog *gd, gpointer data)
 	file_data_multiple_free(fdm);
 }
 
+/* rename option */
+
+static void file_util_move_multiple_rename_auto_cb(GtkWidget *widget, gpointer data)
+{
+	GenericDialog *gd = data;
+	FileDataMult *fdm;
+
+	fdm = gd->data;
+
+	fdm->rename_auto = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	gtk_widget_set_sensitive(fdm->rename_box, !fdm->rename_auto);
+	gtk_widget_set_sensitive(fdm->rename_entry, (!fdm->rename_auto && fdm->rename));
+
+	gtk_widget_set_sensitive(fdm->yes_all_button, (fdm->rename_auto || !fdm->rename));
+}
+
+static void file_util_move_multiple_rename_cb(GtkWidget *widget, gpointer data)
+{
+	GenericDialog *gd = data;
+	FileDataMult *fdm;
+
+	fdm = gd->data;
+
+	fdm->rename = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	gtk_widget_set_sensitive(fdm->rename_entry, fdm->rename);
+	gtk_widget_set_sensitive(fdm->yes_all_button, !fdm->rename);
+
+	if (fdm->rename) gtk_widget_grab_focus(fdm->rename_entry);
+}
+
 static void file_util_move_multiple(FileDataMult *fdm)
 {
 	while (fdm->dest || fdm->source_next)
@@ -335,16 +420,51 @@ static void file_util_move_multiple(FileDataMult *fdm)
 			GenericDialog *gd;
 			gchar *text;
 
+			GtkWidget *hbox;
+
 			text = g_strdup_printf(_("Overwrite file:\n %s\n with:\n %s"), fdm->dest, fdm->source);
 			gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
 						file_util_move_multiple_cancel_cb, fdm);
 			g_free(text);
 
 			generic_dialog_add(gd, _("Yes"), file_util_move_multiple_ok_cb, TRUE);
-			generic_dialog_add(gd, _("Yes to all"), file_util_move_multiple_all_cb, FALSE);
+			fdm->yes_all_button = generic_dialog_add(gd, _("Yes to all"),
+								 file_util_move_multiple_all_cb, FALSE);
 			generic_dialog_add(gd, _("Skip"), file_util_move_multiple_skip_cb, FALSE);
 			generic_dialog_add_images(gd, fdm->dest, fdm->source);
 
+			/* rename option */
+
+			fdm->rename = FALSE;
+			fdm->rename_all = FALSE;
+			fdm->rename_auto = FALSE;
+
+			hbox = gtk_hbox_new(FALSE, 5);
+			gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
+			gtk_widget_show(hbox);
+
+			fdm->rename_auto_box = gtk_check_button_new_with_label(_("Auto rename"));
+			gtk_signal_connect(GTK_OBJECT(fdm->rename_auto_box), "clicked",
+					   GTK_SIGNAL_FUNC(file_util_move_multiple_rename_auto_cb), gd);
+			gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_auto_box, FALSE, FALSE, 0);
+			gtk_widget_show(fdm->rename_auto_box);
+
+			hbox = gtk_hbox_new(FALSE, 5);
+			gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
+			gtk_widget_show(hbox);
+
+			fdm->rename_box = gtk_check_button_new_with_label(_("Rename"));
+			gtk_signal_connect(GTK_OBJECT(fdm->rename_box), "clicked",
+					   GTK_SIGNAL_FUNC(file_util_move_multiple_rename_cb), gd);
+			gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_box, FALSE, FALSE, 0);
+			gtk_widget_show(fdm->rename_box);
+
+			fdm->rename_entry = gtk_entry_new();
+			gtk_entry_set_text(GTK_ENTRY(fdm->rename_entry), filename_from_path(fdm->dest));
+			gtk_widget_set_sensitive(fdm->rename_entry, FALSE);
+			gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_entry, TRUE, TRUE, 0);
+			gtk_widget_show(fdm->rename_entry);
+
 			gtk_widget_show(gd->dialog);
 			return;
 			}
@@ -358,16 +478,36 @@ static void file_util_move_multiple(FileDataMult *fdm)
 				}
 			else
 				{
-				if (fdm->copy)
+				gint try = TRUE;
+
+				if (fdm->confirm_all && fdm->rename_all && isfile(fdm->dest))
 					{
-					success = copy_file(fdm->source, fdm->dest);
-					}
-				else
-					{
-					if (move_file(fdm->source, fdm->dest))
+					gchar *buf;
+					buf = unique_filename_simple(fdm->dest);
+					if (buf)
 						{
-						success = TRUE;
-						file_maint_moved(fdm->source, fdm->dest, fdm->source_list);
+						g_free(fdm->dest);
+						fdm->dest = buf;
+						}
+					else
+						{
+						try = FALSE;
+						}
+					}
+
+				if (try)
+					{
+					if (fdm->copy)
+						{
+						success = copy_file(fdm->source, fdm->dest);
+						}
+					else
+						{
+						if (move_file(fdm->source, fdm->dest))
+							{
+							success = TRUE;
+							file_maint_moved(fdm->source, fdm->dest, fdm->source_list);
+							}
 						}
 					}
 				}
@@ -433,6 +573,46 @@ static void file_util_move_single_ok_cb(GenericDialog *gd, gpointer data)
 	FileDataSingle *fds = data;
 
 	fds->confirmed = TRUE;
+
+	if (fds->rename_auto)
+		{
+		gchar *buf;
+
+		buf = unique_filename_simple(fds->dest);
+		if (buf)
+			{
+			g_free(fds->dest);
+			fds->dest = buf;
+			}
+		else
+			{
+			/* unique failed? well, return to the overwrite prompt :( */
+			fds->confirmed = FALSE;
+			}
+		}
+	else if (fds->rename)
+		{
+		const gchar *name;
+
+		name = gtk_entry_get_text(GTK_ENTRY(fds->rename_entry));
+		if (strlen(name) == 0 ||
+		    strcmp(name, filename_from_path(fds->source)) == 0)
+			{
+			fds->confirmed = FALSE;
+			}
+		else
+			{
+			gchar *base;
+
+			base = remove_level_from_path(fds->dest);
+			g_free(fds->dest);
+			fds->dest = concat_dir_and_file(base, name);
+			fds->confirmed = !isname(fds->dest);
+
+			g_free(base);
+			}
+		}
+
 	file_util_move_single(fds);
 }
 
@@ -443,6 +623,31 @@ static void file_util_move_single_cancel_cb(GenericDialog *gd, gpointer data)
 	file_data_single_free(fds);
 }
 
+static void file_util_move_single_rename_auto_cb(GtkWidget *widget, gpointer data)
+{
+	GenericDialog *gd = data;
+	FileDataSingle *fds;
+
+	fds = gd->data;
+
+	fds->rename_auto = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	gtk_widget_set_sensitive(fds->rename_box, !fds->rename_auto);
+	gtk_widget_set_sensitive(fds->rename_entry, (!fds->rename_auto && fds->rename));
+}
+
+static void file_util_move_single_rename_cb(GtkWidget *widget, gpointer data)
+{
+	GenericDialog *gd = data;
+	FileDataSingle *fds;
+
+	fds = gd->data;
+
+	fds->rename = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	gtk_widget_set_sensitive(fds->rename_entry, fds->rename);
+
+	if (fds->rename) gtk_widget_grab_focus(fds->rename_entry);
+}
+
 static void file_util_move_single(FileDataSingle *fds)
 {
 	if (fds->dest && fds->source && strcmp(fds->dest, fds->source) == 0)
@@ -455,6 +660,8 @@ static void file_util_move_single(FileDataSingle *fds)
 		GenericDialog *gd;
 		gchar *text;
 
+		GtkWidget *hbox;
+
 		text = g_strdup_printf(_("Overwrite file:\n%s\n with:\n%s"), fds->dest, fds->source);
 		gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
 					file_util_move_single_cancel_cb, fds);
@@ -463,6 +670,37 @@ static void file_util_move_single(FileDataSingle *fds)
 		generic_dialog_add(gd, _("Overwrite"), file_util_move_single_ok_cb, TRUE);
 		generic_dialog_add_images(gd, fds->dest, fds->source);
 
+		/* rename option */
+
+		fds->rename = FALSE;
+		fds->rename_auto = FALSE;
+
+		hbox = gtk_hbox_new(FALSE, 5);
+		gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
+		gtk_widget_show(hbox);
+
+		fds->rename_auto_box = gtk_check_button_new_with_label(_("Auto rename"));
+		gtk_signal_connect(GTK_OBJECT(fds->rename_auto_box), "clicked",
+				   GTK_SIGNAL_FUNC(file_util_move_single_rename_auto_cb), gd);
+		gtk_box_pack_start(GTK_BOX(hbox), fds->rename_auto_box, FALSE, FALSE, 0);
+		gtk_widget_show(fds->rename_auto_box);
+
+		hbox = gtk_hbox_new(FALSE, 5);
+		gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
+		gtk_widget_show(hbox);
+
+		fds->rename_box = gtk_check_button_new_with_label(_("Rename"));
+		gtk_signal_connect(GTK_OBJECT(fds->rename_box), "clicked",
+				   GTK_SIGNAL_FUNC(file_util_move_single_rename_cb), gd);
+		gtk_box_pack_start(GTK_BOX(hbox), fds->rename_box, FALSE, FALSE, 0);
+		gtk_widget_show(fds->rename_box);
+
+		fds->rename_entry = gtk_entry_new();
+		gtk_entry_set_text(GTK_ENTRY(fds->rename_entry), filename_from_path(fds->dest));
+		gtk_widget_set_sensitive(fds->rename_entry, FALSE);
+		gtk_box_pack_start(GTK_BOX(hbox), fds->rename_entry, TRUE, TRUE, 0);
+		gtk_widget_show(fds->rename_entry);
+
 		gtk_widget_show(gd->dialog);
 		return;
 		}
