@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -35,6 +35,12 @@ void file_util_copy(const gchar *source_path, GList *source_list, const gchar *d
 void file_util_rename(const gchar *source_path, GList *source_list);
 void file_util_create_dir(const gchar *path);
 
+/* these avoid the location entry dialog, list must be files only and
+ * dest_path must be a valid directory path
+*/
+void file_util_move_simple(GList *list, const gchar *dest_path);
+void file_util_copy_simple(GList *list, const gchar *dest_path);
+
 
 #endif
 
