@@ -140,8 +140,8 @@ static GtkWidget *vdlist_drop_menu(ViewDirList *vdl, gint active)
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add_sensitive(menu, _("Copy"), active, vdlist_drop_menu_copy_cb, vdl);
-	menu_item_add_sensitive(menu, _("Move"), active, vdlist_drop_menu_move_cb, vdl);
+	menu_item_add_sensitive(menu, _("Copy"), active, GTK_SIGNAL_FUNC(vdlist_drop_menu_copy_cb), vdl);
+	menu_item_add_sensitive(menu, _("Move"), active, GTK_SIGNAL_FUNC(vdlist_drop_menu_move_cb), vdl);
 
 	menu_item_add_divider(menu);
 	menu_item_add(menu, _("Cancel"), NULL, vdl);
@@ -293,28 +293,34 @@ static GtkWidget *vdlist_pop_menu(ViewDirList *vdl, gint row)
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add_sensitive(menu, _("Slideshow"), active, vdlist_pop_menu_slide_cb, vdl);
-	menu_item_add_sensitive(menu, _("Slideshow recursive"), active, vdlist_pop_menu_slide_rec_cb, vdl);
+	menu_item_add_sensitive(menu, _("Slideshow"), active,
+				GTK_SIGNAL_FUNC(vdlist_pop_menu_slide_cb), vdl);
+	menu_item_add_sensitive(menu, _("Slideshow recursive"), active,
+				GTK_SIGNAL_FUNC(vdlist_pop_menu_slide_rec_cb), vdl);
 
 	menu_item_add_divider(menu);
-	menu_item_add_sensitive(menu, _("Find duplicates..."), active, vdlist_pop_menu_dupe_cb, vdl);
-	menu_item_add_sensitive(menu, _("Find duplicates recursive..."), active, vdlist_pop_menu_dupe_rec_cb, vdl);
+	menu_item_add_sensitive(menu, _("Find duplicates..."), active,
+				GTK_SIGNAL_FUNC(vdlist_pop_menu_dupe_cb), vdl);
+	menu_item_add_sensitive(menu, _("Find duplicates recursive..."), active,
+				GTK_SIGNAL_FUNC(vdlist_pop_menu_dupe_rec_cb), vdl);
 
 	menu_item_add_divider(menu);
 
 	/* check using . (always row 0) */
 	active = (access(vdlist_row_get_path(vdl, 0), W_OK | X_OK) == 0);
-	menu_item_add_sensitive(menu, _("New folder..."), active, vdlist_pop_menu_new_cb, vdl);
+	menu_item_add_sensitive(menu, _("New folder..."), active,
+				GTK_SIGNAL_FUNC(vdlist_pop_menu_new_cb), vdl);
 
 	/* ignore . and .. */
 	active = (row >= 2 && active &&
 		  access(vdlist_row_get_path(vdl, vdl->click_row), W_OK | X_OK) == 0);
-	menu_item_add_sensitive(menu, _("Rename..."), active, vdlist_pop_menu_rename_cb, vdl);
+	menu_item_add_sensitive(menu, _("Rename..."), active,
+				GTK_SIGNAL_FUNC(vdlist_pop_menu_rename_cb), vdl);
 
 	menu_item_add_divider(menu);
 	menu_item_add_check(menu, _("View as tree"), FALSE,
-			    vdlist_pop_menu_tree_cb, vdl);
-	menu_item_add(menu, _("Refresh"), vdlist_pop_menu_refresh_cb, vdl);
+			    GTK_SIGNAL_FUNC(vdlist_pop_menu_tree_cb), vdl);
+	menu_item_add(menu, _("Refresh"), GTK_SIGNAL_FUNC(vdlist_pop_menu_refresh_cb), vdl);
 
 	return menu;
 }
@@ -443,11 +449,21 @@ static void vdlist_dnd_drop_receive(GtkWidget *widget,
 		}
 }
 
+static void vdlist_dnd_drop_scroll_cancel(ViewDirList *vdl)
+{
+	if (vdl->drop_scroll_id != -1) gtk_timeout_remove(vdl->drop_scroll_id);
+	vdl->drop_scroll_id = -1;
+}
+
 static gint vdlist_dnd_drop_scroll_cb(gpointer data)
 {
 	ViewDirList *vdl = data;
 
-	if (vdl->drop_row == -1 || vdl->drop_list) return FALSE;
+	if (vdl->drop_row == -1 || vdl->drop_list)
+		{
+		vdl->drop_scroll_id = -1;
+		return FALSE;
+		}
 	if (vdl->drop_row < 1 || vdl->drop_row >= GTK_CLIST(vdl->clist)->rows - 1) return TRUE;
 
 	vdlist_set_prelight(vdl, vdl->drop_row, FALSE);
@@ -497,9 +513,9 @@ static gint vdlist_dnd_drop_motion(GtkWidget *widget, GdkDragContext *context,
 		}
 
 	/* delayed drop scrolling */
-	if (vdl->drop_row == -1 && row != -1)
+	if (vdl->drop_row == -1 && row != -1 && vdl->drop_scroll_id == -1)
 		{
-		gtk_timeout_add(100, vdlist_dnd_drop_scroll_cb, vdl);
+		vdl->drop_scroll_id = gtk_timeout_add(100, vdlist_dnd_drop_scroll_cb, vdl);
 		}
 	vdl->drop_row = row;
 
@@ -623,10 +639,23 @@ gint vdlist_set_path(ViewDirList *vdl, const gchar *path)
 {
 	gint ret;
 	FileData *fd;
+	gchar *old_path = NULL;
 
 	if (!path) return FALSE;
 	if (vdl->path && strcmp(path, vdl->path) == 0) return TRUE;
 
+	if (vdl->path)
+		{
+		gchar *base;
+
+		base = remove_level_from_path(vdl->path);
+		if (strcmp(base, path) == 0)
+			{
+			old_path = g_strdup(filename_from_path(vdl->path));
+			}
+		g_free(base);
+		}
+
 	g_free(vdl->path);
 	vdl->path = g_strdup(path);
 
@@ -650,6 +679,33 @@ gint vdlist_set_path(ViewDirList *vdl, const gchar *path)
 	vdl->list = g_list_prepend(vdl->list, fd);
 
 	vdlist_populate(vdl);
+
+	if (old_path)
+		{
+		/* scroll to make last path visible */
+		gint row;
+		gint found;
+		GList *work;
+
+		row = -1;
+		found = FALSE;
+		work = vdl->list;
+		while (work && !found)
+			{
+			FileData *fd;
+
+			fd = work->data;
+			work = work->next;
+
+			row++;
+			if (strcmp(old_path, fd->name) == 0) found = TRUE;
+			}
+
+		if (found) gtk_clist_moveto(GTK_CLIST(vdl->clist), row, 0, 0.5, 0.0);
+
+		g_free(old_path);
+		}
+
 	return ret;
 }
 
@@ -712,6 +768,8 @@ static void vdlist_destroy_cb(GtkWidget *widget, gpointer data)
 		gtk_signal_disconnect_by_data(GTK_OBJECT(vdl->popup), vdl);
 		}
 
+	vdlist_dnd_drop_scroll_cancel(vdl);
+
 	path_list_free(vdl->drop_list);
 
 	folder_icons_free(vdl->pf);
@@ -734,18 +792,21 @@ ViewDirList *vdlist_new(const gchar *path)
 	vdl->drop_row = -1;
 	vdl->drop_list = NULL;
 
+	vdl->drop_scroll_id = -1;
+
 	vdl->popup = NULL;
 
 	vdl->widget = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(vdl->widget),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-	gtk_signal_connect(GTK_OBJECT(vdl->widget), "destroy", vdlist_destroy_cb, vdl);
+	gtk_signal_connect(GTK_OBJECT(vdl->widget), "destroy",
+			   GTK_SIGNAL_FUNC(vdlist_destroy_cb), vdl);
 
 	vdl->clist = gtk_clist_new(1);
 	gtk_signal_connect(GTK_OBJECT(vdl->clist), "button_press_event",
-			   (GtkSignalFunc) vdlist_press_cb, vdl);
+			   GTK_SIGNAL_FUNC(vdlist_press_cb), vdl);
 	gtk_signal_connect(GTK_OBJECT(vdl->clist), "select_row",
-			   (GtkSignalFunc)vdlist_select_cb, vdl);
+			   GTK_SIGNAL_FUNC(vdlist_select_cb), vdl);
 	gtk_clist_column_titles_passive(GTK_CLIST(vdl->clist));
 	gtk_container_add(GTK_CONTAINER(vdl->widget), vdl->clist);
 	gtk_widget_show(vdl->clist);
