@@ -209,8 +209,8 @@ static GtkWidget *vdtree_drop_menu(ViewDirTree *vdt, gint active)
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add_sensitive(menu, _("Copy"), active, vdtree_drop_menu_copy_cb, vdt);
-	menu_item_add_sensitive(menu, _("Move"), active, vdtree_drop_menu_move_cb, vdt);
+	menu_item_add_sensitive(menu, _("Copy"), active, GTK_SIGNAL_FUNC(vdtree_drop_menu_copy_cb), vdt);
+	menu_item_add_sensitive(menu, _("Move"), active, GTK_SIGNAL_FUNC(vdtree_drop_menu_move_cb), vdt);
 
 	menu_item_add_divider(menu);
 	menu_item_add(menu, _("Cancel"), NULL, vdt);
@@ -363,27 +363,33 @@ static GtkWidget *vdtree_pop_menu(ViewDirTree *vdt, GtkCTreeNode *node)
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add_sensitive(menu, _("Slideshow"), active, vdtree_pop_menu_slide_cb, vdt);
-	menu_item_add_sensitive(menu, _("Slideshow recursive"), active, vdtree_pop_menu_slide_rec_cb, vdt);
+	menu_item_add_sensitive(menu, _("Slideshow"), active,
+				GTK_SIGNAL_FUNC(vdtree_pop_menu_slide_cb), vdt);
+	menu_item_add_sensitive(menu, _("Slideshow recursive"), active,
+				GTK_SIGNAL_FUNC(vdtree_pop_menu_slide_rec_cb), vdt);
 
 	menu_item_add_divider(menu);
-	menu_item_add_sensitive(menu, _("Find duplicates..."), active, vdtree_pop_menu_dupe_cb, vdt);
-	menu_item_add_sensitive(menu, _("Find duplicates recursive..."), active, vdtree_pop_menu_dupe_rec_cb, vdt);
+	menu_item_add_sensitive(menu, _("Find duplicates..."), active,
+				GTK_SIGNAL_FUNC(vdtree_pop_menu_dupe_cb), vdt);
+	menu_item_add_sensitive(menu, _("Find duplicates recursive..."), active,
+				GTK_SIGNAL_FUNC(vdtree_pop_menu_dupe_rec_cb), vdt);
 
 	menu_item_add_divider(menu);
 
 	active = (node &&
 		  access(vdtree_node_get_path(vdt, node), W_OK | X_OK) == 0);
-	menu_item_add_sensitive(menu, _("New folder..."), active, vdtree_pop_menu_new_cb, vdt);
+	menu_item_add_sensitive(menu, _("New folder..."), active,
+				GTK_SIGNAL_FUNC(vdtree_pop_menu_new_cb), vdt);
 
 	active = (active && GTK_CTREE_ROW(node)->parent &&
 		  access(vdtree_node_get_path(vdt, GTK_CTREE_ROW(node)->parent), W_OK | X_OK) == 0);
-	menu_item_add_sensitive(menu, _("Rename..."), active, vdtree_pop_menu_rename_cb, vdt);
+	menu_item_add_sensitive(menu, _("Rename..."), active,
+				GTK_SIGNAL_FUNC(vdtree_pop_menu_rename_cb), vdt);
 
 	menu_item_add_divider(menu);
 	menu_item_add_check(menu, _("View as tree"), TRUE,
-			    vdtree_pop_menu_tree_cb, vdt);
-	menu_item_add(menu, _("Refresh"), vdtree_pop_menu_refresh_cb, vdt);
+			    GTK_SIGNAL_FUNC(vdtree_pop_menu_tree_cb), vdt);
+	menu_item_add(menu, _("Refresh"), GTK_SIGNAL_FUNC(vdtree_pop_menu_refresh_cb), vdt);
 
 	return menu;
 }
@@ -541,11 +547,21 @@ static void vdtree_dnd_drop_expand(ViewDirTree *vdt, GtkCTreeNode *node)
 	vdt->drop_expand_id = gtk_timeout_add(1000, vdtree_dnd_drop_expand_cb, vdt);
 }
 
+static void vdtree_dnd_drop_scroll_cancel(ViewDirTree *vdt)
+{
+        if (vdt->drop_scroll_id != -1) gtk_timeout_remove(vdt->drop_scroll_id);
+        vdt->drop_scroll_id = -1;
+}
+
 static gint vdtree_dnd_drop_scroll_cb(gpointer data)
 {
 	ViewDirTree *vdt = data;
 
-	if (vdt->drop_row == -1 || vdt->drop_list) return FALSE;
+	if (vdt->drop_row == -1 || vdt->drop_list)
+		{
+		vdt->drop_scroll_id = -1;
+		return FALSE;
+		}
 
 	if (vdt->drop_row < 1 || vdt->drop_row >= GTK_CLIST(vdt->ctree)->rows - 1) return TRUE;
 
@@ -614,9 +630,9 @@ static gint vdtree_dnd_drop_motion(GtkWidget *widget, GdkDragContext *context,
 		}
 
 	/* delayed drop scrolling */
-	if (vdt->drop_row == -1 && row != -1)
+	if (vdt->drop_row == -1 && row != -1 && vdt->drop_scroll_id == -1)
 		{
-		gtk_timeout_add(100, vdtree_dnd_drop_scroll_cb, vdt);
+		vdt->drop_scroll_id = gtk_timeout_add(100, vdtree_dnd_drop_scroll_cb, vdt);
 		}
 	vdt->drop_row = row;
 
@@ -629,6 +645,7 @@ static void vdtree_dnd_drop_leave(GtkWidget *widget, GdkDragContext *context, gu
 
 	if (vdt->drop_node != vdt->click_node) vdtree_set_node_color(vdt, vdt->drop_node, FALSE, FALSE);
 
+	vdt->drop_node = NULL;
 	vdt->drop_row = -1;
 
 	vdtree_dnd_drop_expand_cancel(vdt);
@@ -640,17 +657,17 @@ static void vdtree_dnd_init(ViewDirTree *vdt)
 			    dnd_file_drag_types, dnd_file_drag_types_count,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK);
 	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "drag_data_get",
-			    vdtree_dnd_get, vdt);
+			   GTK_SIGNAL_FUNC(vdtree_dnd_get), vdt);
 	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "drag_begin",
-			    vdtree_dnd_begin, vdt);
+			   GTK_SIGNAL_FUNC(vdtree_dnd_begin), vdt);
 	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "drag_end",
-			    vdtree_dnd_end, vdt);
+			   GTK_SIGNAL_FUNC(vdtree_dnd_end), vdt);
 	gtk_drag_dest_set(vdt->ctree,
 			GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
 			vdtree_dnd_drop_types, vdtree_dnd_drop_types_count,
 			GDK_ACTION_MOVE | GDK_ACTION_COPY);
 	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "drag_data_received",
-			vdtree_dnd_drop_receive, vdt);
+			GTK_SIGNAL_FUNC(vdtree_dnd_drop_receive), vdt);
 	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "drag_motion",
 			GTK_SIGNAL_FUNC(vdtree_dnd_drop_motion), vdt);
 	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "drag_leave",
@@ -935,6 +952,8 @@ static gint vdtree_populate_path_by_node(ViewDirTree *vdt, GtkCTreeNode *node, g
 		if (!isdir(nd->fd->path))
 			{
 			gtk_ctree_remove_node(GTK_CTREE(vdt->ctree), node);
+			if (vdt->click_node == node) vdt->click_node = NULL;
+			if (vdt->drop_node == node) vdt->drop_node = NULL;
 			return FALSE;
 			}
 		if (!force && filetime(nd->fd->path) == nd->fd->date) return TRUE;
@@ -992,6 +1011,8 @@ static gint vdtree_populate_path_by_node(ViewDirTree *vdt, GtkCTreeNode *node, g
 	while (work)
 		{
 		gtk_ctree_remove_node(GTK_CTREE(vdt->ctree), GTK_CTREE_NODE(work->data));
+		if (vdt->click_node == node) vdt->click_node = NULL;
+		if (vdt->drop_node == node) vdt->drop_node = NULL;
 		work = work->next;
 		}
 
@@ -1178,7 +1199,7 @@ static void vdtree_select_cb(GtkWidget *widget, GtkCTreeNode *node, gint column,
 		}
 }
 
-static void vdtree_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint vdtree_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	ViewDirTree *vdt = data;
 	gint row = -1;
@@ -1204,6 +1225,8 @@ static void vdtree_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer
 		gtk_menu_popup(GTK_MENU(vdt->popup), NULL, NULL, NULL, NULL,
 			       bevent->button, bevent->time);
 		}
+
+	return FALSE;
 }
 
 /*
@@ -1222,6 +1245,7 @@ static void vdtree_destroy_cb(GtkWidget *widget, gpointer data)
 		}
 
 	vdtree_dnd_drop_expand_cancel(vdt);
+	vdtree_dnd_drop_scroll_cancel(vdt);
 	path_list_free(vdt->drop_list);
 
 	folder_icons_free(vdt->pf);
@@ -1241,6 +1265,7 @@ ViewDirTree *vdtree_new(const gchar *path, gint expand)
 
 	vdt->drop_node = NULL;
 	vdt->drop_list = NULL;
+	vdt->drop_scroll_id = -1;
 	vdt->drop_expand_id = -1;
 	vdt->drop_row = -1;
 
@@ -1249,7 +1274,8 @@ ViewDirTree *vdtree_new(const gchar *path, gint expand)
 	vdt->widget = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(vdt->widget),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-	gtk_signal_connect(GTK_OBJECT(vdt->widget), "destroy", vdtree_destroy_cb, vdt);
+	gtk_signal_connect(GTK_OBJECT(vdt->widget), "destroy",
+			   GTK_SIGNAL_FUNC(vdtree_destroy_cb), vdt);
 
 	vdt->ctree = gtk_ctree_new(1, 0);
 	gtk_ctree_set_indent(GTK_CTREE(vdt->ctree), VDTREE_INDENT); /* sorry, had to hard code it */
@@ -1257,11 +1283,15 @@ ViewDirTree *vdtree_new(const gchar *path, gint expand)
 	gtk_clist_set_selection_mode(GTK_CLIST(vdt->ctree), GTK_SELECTION_BROWSE);
 	gtk_clist_set_column_auto_resize(GTK_CLIST(vdt->ctree), 0 , TRUE);
 
-	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "tree_expand", vdtree_expand_cb, vdt);
-	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "tree_collapse", vdtree_collapse_cb, vdt);
+	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "tree_expand",
+			   GTK_SIGNAL_FUNC(vdtree_expand_cb), vdt);
+	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "tree_collapse",
+			   GTK_SIGNAL_FUNC(vdtree_collapse_cb), vdt);
 
-	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "button_press_event", vdtree_press_cb, vdt);
-	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "tree_select_row", vdtree_select_cb, vdt);
+	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "button_press_event",
+			   GTK_SIGNAL_FUNC(vdtree_press_cb), vdt);
+	gtk_signal_connect(GTK_OBJECT(vdt->ctree), "tree_select_row",
+			   GTK_SIGNAL_FUNC(vdtree_select_cb), vdt);
 
 	gtk_container_add(GTK_CONTAINER(vdt->widget), vdt->ctree);
 	gtk_clist_set_auto_sort(GTK_CLIST(vdt->ctree), TRUE);
