@@ -17,6 +17,7 @@
 #include "collect-table.h"
 #include "dnd.h"
 #include "img-view.h"
+#include "info.h"
 #include "filelist.h"
 #include "layout.h"
 #include "layout_image.h"
@@ -167,6 +168,13 @@ static void vficon_pop_menu_edit_cb(GtkWidget *widget, gpointer data)
 	path_list_free(list);
 }
 
+static void vficon_pop_menu_info_cb(GtkWidget *widget, gpointer data)
+{
+	ViewFileIcon *vfi = data;
+
+	info_window_new(NULL, vficon_pop_menu_file_list(vfi));
+}
+
 static void vficon_pop_menu_view_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileIcon *vfi = data;
@@ -261,33 +269,36 @@ static GtkWidget *vficon_pop_menu(ViewFileIcon *vfi, gint active)
 	submenu_add_edit(menu, &item, vficon_pop_menu_edit_cb, vfi);
 	gtk_widget_set_sensitive(item, active);
 
-	item = menu_item_add(menu, _("View in new window"), vficon_pop_menu_view_cb, vfi);
+	item = menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(vficon_pop_menu_info_cb), vfi);
+	gtk_widget_set_sensitive(item, active);
+
+	item = menu_item_add(menu, _("View in new window"), GTK_SIGNAL_FUNC(vficon_pop_menu_view_cb), vfi);
 	gtk_widget_set_sensitive(item, active);
 
 	menu_item_add_divider(menu);
-	item = menu_item_add(menu, _("Copy..."), vficon_pop_menu_copy_cb, vfi);
+	item = menu_item_add(menu, _("Copy..."), GTK_SIGNAL_FUNC(vficon_pop_menu_copy_cb), vfi);
 	gtk_widget_set_sensitive(item, active);
-	item = menu_item_add(menu, _("Move..."), vficon_pop_menu_move_cb, vfi);
+	item = menu_item_add(menu, _("Move..."), GTK_SIGNAL_FUNC(vficon_pop_menu_move_cb), vfi);
 	gtk_widget_set_sensitive(item, active);
-	item = menu_item_add(menu, _("Rename..."), vficon_pop_menu_rename_cb, vfi);
+	item = menu_item_add(menu, _("Rename..."), GTK_SIGNAL_FUNC(vficon_pop_menu_rename_cb), vfi);
 	gtk_widget_set_sensitive(item, active);
-	item = menu_item_add(menu, _("Delete..."), vficon_pop_menu_delete_cb, vfi);
+	item = menu_item_add(menu, _("Delete..."), GTK_SIGNAL_FUNC(vficon_pop_menu_delete_cb), vfi);
 	gtk_widget_set_sensitive(item, active);
 
 	menu_item_add_divider(menu);
 
-	submenu = submenu_add_sort(NULL, vficon_pop_menu_sort_cb, vfi,
+	submenu = submenu_add_sort(NULL, GTK_SIGNAL_FUNC(vficon_pop_menu_sort_cb), vfi,
 				   FALSE, FALSE, TRUE, vfi->sort_method);
 	menu_item_add_divider(submenu);
 	menu_item_add_check(submenu, _("Ascending"), vfi->sort_ascend,
-			    vficon_pop_menu_sort_ascend_cb, vfi);
+			    GTK_SIGNAL_FUNC(vficon_pop_menu_sort_ascend_cb), vfi);
 
 	item = menu_item_add(menu, _("Sort"), NULL, NULL);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
 	menu_item_add_check(menu, _("View as icons"), TRUE,
-			    vficon_pop_menu_list_cb, vfi);
-	menu_item_add(menu, _("Refresh"), vficon_pop_menu_refresh_cb, vfi);
+			    GTK_SIGNAL_FUNC(vficon_pop_menu_list_cb), vfi);
+	menu_item_add(menu, _("Refresh"), GTK_SIGNAL_FUNC(vficon_pop_menu_refresh_cb), vfi);
 
 	return menu;
 }
@@ -542,11 +553,11 @@ static void vficon_dnd_init(ViewFileIcon *vfi)
 			    dnd_file_drag_types, dnd_file_drag_types_count,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	gtk_signal_connect(GTK_OBJECT(vfi->clist), "drag_data_get",
-			   vficon_dnd_get, vfi);
+			   GTK_SIGNAL_FUNC(vficon_dnd_get), vfi);
 	gtk_signal_connect(GTK_OBJECT(vfi->clist), "drag_begin",
-			   vficon_dnd_begin, vfi);
+			   GTK_SIGNAL_FUNC(vficon_dnd_begin), vfi);
 	gtk_signal_connect(GTK_OBJECT(vfi->clist), "drag_end",
-			   vficon_dnd_end, vfi);
+			   GTK_SIGNAL_FUNC(vficon_dnd_end), vfi);
 }
 
 /*
@@ -1390,16 +1401,36 @@ static gint vficon_add_row(ViewFileIcon *vfi)
 	return row;
 }
 
-static void vficon_populate(ViewFileIcon *vfi, gint resize)
+#define CLIST_FRAME_SIZE 4
+
+static void vficon_populate(ViewFileIcon *vfi, gint resize, gint keep_position)
 {
 	GtkCList *clist;
 	gint col, row;
 	GList *work;
+	gint visible_row = -1;
+	gfloat visible_offset = 0.0;
 
 	vficon_verify_selections(vfi);
 
 	clist = GTK_CLIST(vfi->clist);
 
+	if (keep_position)
+		{
+		gtk_clist_get_selection_info(clist, 0, 0, &visible_row, 0);
+		if (clist->row_height > 0 && clist->clist_window_height > 0 && visible_row < clist->rows - 1)
+			{
+			gint icon_offset;
+
+			/* ugly, but it maintains exact scroll offset when refreshing */
+
+			icon_offset = clist->row_height + (clist->voffset % (clist->row_height + 1));
+			visible_offset = (gfloat)icon_offset / (clist->clist_window_height - clist->row_height - CLIST_FRAME_SIZE);
+
+			visible_row++;
+			}
+		}
+
 	gtk_clist_freeze(clist);
 	gtk_clist_clear(clist);
 
@@ -1443,6 +1474,11 @@ static void vficon_populate(ViewFileIcon *vfi, gint resize)
 		if (col >= vfi->columns) col = 0;
 		}
 
+	if (keep_position && visible_row > 0)
+		{
+		gtk_clist_moveto(clist, visible_row, -1, visible_offset, 0.0);
+		}
+
 	gtk_clist_thaw(clist);
 
 	vfi->rows = row + 1;
@@ -1463,7 +1499,7 @@ static void vficon_populate_at_new_size(ViewFileIcon *vfi, gint w, gint h)
 
 	vfi->columns = new_cols;
 
-	vficon_populate(vfi, TRUE);
+	vficon_populate(vfi, TRUE, TRUE);
 
 	if (debug) printf("col tab pop cols=%d rows=%d\n", vfi->columns, vfi->rows);
 }
@@ -1488,6 +1524,7 @@ static void vficon_sync(ViewFileIcon *vfi)
 		if (vficon_find_position(vfi, fd, &r, &c))
 			{
 			vficon_add_icon(vfi, fd, r, c);
+			c++;
 			}
 		work = work->next;
 		}
@@ -1498,9 +1535,8 @@ static void vficon_sync(ViewFileIcon *vfi)
 		vfi->rows = 0;
 		gtk_clist_remove(GTK_CLIST(vfi->clist), r);
 		}
-	else if (c < vfi->columns - 1)
+	else if (c < vfi->columns)
 		{
-		c++;
 		while (c < vfi->columns)
 			{
 			vficon_add_icon(vfi, NULL, r, c);
@@ -1859,13 +1895,15 @@ GList *vficon_get_list(ViewFileIcon *vfi)
  *-----------------------------------------------------------------------------
  */
 
-gint vficon_refresh(ViewFileIcon *vfi)
+static gint vficon_refresh_real(ViewFileIcon *vfi, gint keep_position)
 {
 	gint ret = TRUE;
 	GList *old_list;
 	GList *work;
+	FileData *focus_fd;
+	gint fr, fc;
 
-	/* FIXME: vficon_refresh does not update icons if file changes */
+	focus_fd = vficon_find_info(vfi, vfi->focus_row, vfi->focus_column);
 
 	old_list = vfi->list;
 	vfi->list = NULL;
@@ -1905,13 +1943,24 @@ gint vficon_refresh(ViewFileIcon *vfi)
 
 	vfi->list = filelist_sort(vfi->list, vfi->sort_method, vfi->sort_ascend);
 
-	vficon_populate(vfi, TRUE);
+	vficon_populate(vfi, TRUE, keep_position);
+
+	/* attempt to keep focus on same icon when refreshing */
+	if (focus_fd && vficon_find_position(vfi, focus_fd, &fr, &fc))
+		{
+		vficon_move_focus(vfi, fr, fc, FALSE);
+		}
 
 	iconlist_free(old_list);
 
 	return ret;
 }
 
+gint vficon_refresh(ViewFileIcon *vfi)
+{
+	return vficon_refresh_real(vfi, TRUE);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * base
@@ -1935,7 +1984,7 @@ gint vficon_set_path(ViewFileIcon *vfi, const gchar *path)
 	vfi->list = NULL;
 
 	/* NOTE: populate will clear the clist for us */
-	ret = vficon_refresh(vfi);
+	ret = vficon_refresh_real(vfi, FALSE);
 
 	vfi->focus_fd = NULL;
 	vficon_move_focus(vfi, 0, 0, FALSE);
@@ -1952,6 +2001,8 @@ static void vficon_destroy_cb(GtkWidget *widget, gpointer data)
 		gtk_signal_disconnect_by_data(GTK_OBJECT(vfi->popup), vfi);
 		}
 
+	tip_unschedule(vfi);
+
 	vficon_thumb_cleanup(vfi);
 
 	g_free(vfi->path);
@@ -1991,7 +2042,8 @@ ViewFileIcon *vficon_new(const gchar *path)
 
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(vfi->widget),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_signal_connect(GTK_OBJECT(vfi->widget), "destroy", vficon_destroy_cb, vfi);
+	gtk_signal_connect(GTK_OBJECT(vfi->widget), "destroy",
+			   GTK_SIGNAL_FUNC(vficon_destroy_cb), vfi);
 
 	vfi->clist = gtk_clist_new(VFICON_MAX_COLUMNS);
 
@@ -2001,12 +2053,14 @@ ViewFileIcon *vficon_new(const gchar *path)
 
 	/* we roll our own selection, connect to select row, then unselect it */
 	gtk_clist_set_selection_mode(GTK_CLIST(vfi->clist), GTK_SELECTION_SINGLE);
-	gtk_signal_connect(GTK_OBJECT(vfi->clist), "select_row", vficon_select_row_cb, vfi);
+	gtk_signal_connect(GTK_OBJECT(vfi->clist), "select_row",
+			   GTK_SIGNAL_FUNC(vficon_select_row_cb), vfi);
 
 	gtk_container_add(GTK_CONTAINER(vfi->widget), vfi->clist);
 	gtk_widget_show(vfi->clist);
 	
-	gtk_signal_connect(GTK_OBJECT(vfi->clist), "size_allocate", vficon_sized_cb, vfi);
+	gtk_signal_connect(GTK_OBJECT(vfi->clist), "size_allocate",
+			   GTK_SIGNAL_FUNC(vficon_sized_cb), vfi);
 
 	gtk_widget_set_events(vfi->clist, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK |
 			      GDK_BUTTON_PRESS_MASK | GDK_LEAVE_NOTIFY_MASK);
@@ -2187,13 +2241,57 @@ void vficon_maint_removed(ViewFileIcon *vfi, const gchar *path, GList *ignore_li
 					new_row = row - 1;
 					}
 				}
-			if (new_row >= 0)
-				{
-				FileData *fdn;
+			}
+		else if (ignore_list)
+			{
+			GList *work;
 
-				fdn = g_list_nth_data(vfi->list, new_row);
-				vficon_select(vfi, fdn);
+			work = vfi->selection;
+			while (work)
+				{
+				FileData *ignore_fd;
+				GList *tmp;
+				gint match = FALSE;
+
+				ignore_fd = work->data;
+				work = work->next;
+
+				tmp = ignore_list;
+				while (tmp && !match)
+					{
+					const gchar *ignore_path;
+
+					ignore_path = tmp->data;
+					tmp = tmp->next;
+
+					if (strcmp(ignore_fd->path, ignore_path) == 0)
+						{
+						match = TRUE;
+						}
+					}
+				if (!match)
+					{
+					new_row = g_list_index(vfi->list, ignore_fd);
+					work = NULL;
+					}
 				}
+			if (new_row == -1)
+				{
+				/* selection all ignored, use closest */
+				new_row = vficon_maint_find_closest(vfi, row, vficon_count(vfi, NULL), ignore_list);
+				}
+			}
+		else
+			{
+			new_row = g_list_index(vfi->list, vfi->selection->data);
+			}
+		if (new_row >= 0)
+			{
+			FileData *fdn;
+
+			fdn = g_list_nth_data(vfi->list, new_row);
+			vficon_select(vfi, fdn);
+			vficon_send_layout_select(vfi, fdn);
 			}
 		}
 
