@@ -15,6 +15,7 @@
 #include "cache_maint.h"
 #include "dnd.h"
 #include "img-view.h"
+#include "info.h"
 #include "layout.h"
 #include "layout_image.h"
 #include "menu.h"
@@ -108,11 +109,11 @@ static void vflist_dnd_init(ViewFileList *vfl)
 			    dnd_file_drag_types, dnd_file_drag_types_count,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	gtk_signal_connect(GTK_OBJECT(vfl->clist), "drag_data_get",
-			    vflist_dnd_get, vfl);
+			   GTK_SIGNAL_FUNC(vflist_dnd_get), vfl);
 	gtk_signal_connect(GTK_OBJECT(vfl->clist), "drag_begin",
-			    vflist_dnd_begin, vfl);
+			   GTK_SIGNAL_FUNC(vflist_dnd_begin), vfl);
 	gtk_signal_connect(GTK_OBJECT(vfl->clist), "drag_end",
-			    vflist_dnd_end, vfl);
+			   GTK_SIGNAL_FUNC(vflist_dnd_end), vfl);
 }
 
 /*
@@ -147,6 +148,13 @@ static void vflist_pop_menu_edit_cb(GtkWidget *widget, gpointer data)
 	path_list_free(list);
 }
 
+static void vflist_pop_menu_info_cb(GtkWidget *widget, gpointer data)
+{
+	ViewFileList *vfl = data;
+
+	info_window_new(NULL, vflist_pop_menu_file_list(vfl));
+}
+
 static void vflist_pop_menu_view_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileList *vfl = data;
@@ -253,33 +261,36 @@ static GtkWidget *vflist_pop_menu(ViewFileList *vfl, gint active)
 	submenu_add_edit(menu, &item, vflist_pop_menu_edit_cb, vfl);
 	gtk_widget_set_sensitive(item, active);
 
-	item = menu_item_add(menu, _("View in new window"), vflist_pop_menu_view_cb, vfl);
+	item = menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(vflist_pop_menu_info_cb), vfl);
+	gtk_widget_set_sensitive(item, active);
+
+	item = menu_item_add(menu, _("View in new window"), GTK_SIGNAL_FUNC(vflist_pop_menu_view_cb), vfl);
 	gtk_widget_set_sensitive(item, active);
 
 	menu_item_add_divider(menu);
-	item = menu_item_add(menu, _("Copy..."), vflist_pop_menu_copy_cb, vfl);
+	item = menu_item_add(menu, _("Copy..."), GTK_SIGNAL_FUNC(vflist_pop_menu_copy_cb), vfl);
 	gtk_widget_set_sensitive(item, active);
-	item = menu_item_add(menu, _("Move..."), vflist_pop_menu_move_cb, vfl);
+	item = menu_item_add(menu, _("Move..."), GTK_SIGNAL_FUNC(vflist_pop_menu_move_cb), vfl);
 	gtk_widget_set_sensitive(item, active);
-	item = menu_item_add(menu, _("Rename..."), vflist_pop_menu_rename_cb, vfl);
+	item = menu_item_add(menu, _("Rename..."), GTK_SIGNAL_FUNC(vflist_pop_menu_rename_cb), vfl);
 	gtk_widget_set_sensitive(item, active);
-	item = menu_item_add(menu, _("Delete..."), vflist_pop_menu_delete_cb, vfl);
+	item = menu_item_add(menu, _("Delete..."), GTK_SIGNAL_FUNC(vflist_pop_menu_delete_cb), vfl);
 	gtk_widget_set_sensitive(item, active);
 
 	menu_item_add_divider(menu);
 
-	submenu = submenu_add_sort(NULL, vflist_pop_menu_sort_cb, vfl,
+	submenu = submenu_add_sort(NULL, GTK_SIGNAL_FUNC(vflist_pop_menu_sort_cb), vfl,
 				   FALSE, FALSE, TRUE, vfl->sort_method);
 	menu_item_add_divider(submenu);
 	menu_item_add_check(submenu, _("Ascending"), vfl->sort_ascend,
-			    vflist_pop_menu_sort_ascend_cb, vfl);
+			    GTK_SIGNAL_FUNC(vflist_pop_menu_sort_ascend_cb), vfl);
 
 	item = menu_item_add(menu, _("Sort"), NULL, NULL);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
 	menu_item_add_check(menu, _("View as icons"), FALSE,
-			    vflist_pop_menu_icons_cb, vfl);
-	menu_item_add(menu, _("Refresh"), vflist_pop_menu_refresh_cb, vfl);
+			    GTK_SIGNAL_FUNC(vflist_pop_menu_icons_cb), vfl);
+	menu_item_add(menu, _("Refresh"), GTK_SIGNAL_FUNC(vflist_pop_menu_refresh_cb), vfl);
 
 	return menu;
 }
@@ -882,8 +893,10 @@ void vflist_select_by_path(ViewFileList *vfl, const gchar *path)
 	GtkCList *clist;
 	gint row;
 
-	clist = GTK_CLIST(vfl->clist);
 	row = vflist_index_by_path(vfl, path);
+	if (row < 0) return;
+
+	clist = GTK_CLIST(vfl->clist);
 
 	if (!vflist_index_is_selected(vfl, row))
 		{
@@ -1015,19 +1028,18 @@ static void vflist_populate_view(ViewFileList *vfl)
 				pixtext = (gtk_clist_get_cell_type(clist, row_p, 0) == GTK_CELL_PIXTEXT);
 				updated = (fd->date != old_fd->date);
 
-				if ((vfl->thumbs_enabled) != (pixtext) || updated)
+				if (vfl->thumbs_enabled)
 					{
-					/* reset thumb for reload */
-					if (vfl->thumbs_enabled)
+					if (!pixtext || updated)
 						{
 						gtk_clist_set_text(clist, row_p, 0, fd->name);
 						gtk_clist_set_shift(clist, row_p, 0, 0, 5 + thumb_max_width);
 						}
-					else
-						{
-						gtk_clist_set_text(clist, row_p, 0, fd->name);
-						gtk_clist_set_shift(clist, row_p, 0, 0, 0);
-						}
+					}
+				else
+					{
+					gtk_clist_set_text(clist, row_p, 0, fd->name);
+					gtk_clist_set_shift(clist, row_p, 0, 0, 0);
 					}
 
 				if (updated)
@@ -1158,7 +1170,8 @@ ViewFileList *vflist_new(const gchar *path, gint thumbs)
 	vfl->widget = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(vfl->widget),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-	gtk_signal_connect(GTK_OBJECT(vfl->widget), "destroy", vflist_destroy_cb, vfl);
+	gtk_signal_connect(GTK_OBJECT(vfl->widget), "destroy",
+			   GTK_SIGNAL_FUNC(vflist_destroy_cb), vfl);
 
 	vfl->clist = gtk_clist_new(3);
 
@@ -1396,4 +1409,3 @@ void vflist_maint_moved(ViewFileList *vfl, const gchar *source, const gchar *des
 	g_free(buf);
 }
 
-
